# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.cluster import AttemptedVmInfo
from pyVmomi.vim.cluster import NotAttemptedVmInfo
from pyVmomi.vim.cluster import Recommendation

class PowerOnVmResult(DynamicData):
   attempted: list[AttemptedVmInfo] = []
   notAttempted: list[NotAttemptedVmInfo] = []
   recommendations: list[Recommendation] = []



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.cluster import VsanConfigGeneration

from pyVmomi.vim.vsan import RepairTimerInfo
from pyVmomi.vim.vsan import ResyncIopsInfo

class RuntimeStats(DynamicData):
   resyncIopsInfo: Optional[ResyncIopsInfo] = None
   configGeneration: Optional[VsanConfigGeneration] = None
   supportedClusterSize: Optional[int] = None
   repairTimerInfo: Optional[RepairTimerInfo] = None
   componentLimitPerCluster: Optional[int] = None
   maxWitnessClusters: Optional[int] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.VmomiSupport import long

from pyVmomi.vmodl import DynamicData

class VsanNetworkLoadTestResult(DynamicData):
   hostname: str
   status: Optional[str] = None
   client: bool
   bandwidthBps: long
   totalBytes: long
   lostDatagrams: Optional[long] = None
   lossPct: Optional[long] = None
   sentDatagrams: Optional[long] = None
   jitterMs: Optional[float] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.encryption import KeyProviderId
from pyVmomi.vim.encryption import KmipServerInfo

class KmipServerSpec(DynamicData):
   class KeySpec(DynamicData):
      pass

   class WrappingKeyIdKeySpec(KeySpec):
      keyId: str

   class WrappingRotationIntervalKeySpec(KeySpec):
      rotationInterval: Optional[int] = None

   clusterId: KeyProviderId
   info: KmipServerInfo
   password: Optional[str] = None
   defaultKeyType: Optional[str] = None
   keySpec: Optional[KeySpec] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vmodl import DynamicData

class LinkDiscoveryProtocolConfig(DynamicData):
   class ProtocolType(Enum):
      cdp: ClassVar['ProtocolType'] = 'cdp'
      lldp: ClassVar['ProtocolType'] = 'lldp'

   class OperationType(Enum):
      none: ClassVar['OperationType'] = 'none'
      listen: ClassVar['OperationType'] = 'listen'
      advertise: ClassVar['OperationType'] = 'advertise'
      both: ClassVar['OperationType'] = 'both'

   protocol: str
   operation: str



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.VmomiSupport import ManagedObject

from pyVmomi.vmodl import DynamicData

class ServiceManager(ManagedObject):
   class ServiceInfo(DynamicData):
      serviceName: str
      location: list[str] = []
      service: ManagedObject
      description: str

   @property
   def service(self) -> list[ServiceInfo]: ...

   def QueryServiceList(self, serviceName: Optional[str], location: list[str]) -> list[ServiceInfo]: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vmodl import DynamicData
from pyVmomi.vmodl import MethodFault

from pyVmomi.vim.host import VsanPhysicalDiskHealth
from pyVmomi.vim.host import VsanResourceHealth

class VsanPhysicalDiskHealthSummary(DynamicData):
   overallHealth: str
   heapsWithIssues: list[VsanResourceHealth] = []
   slabsWithIssues: list[VsanResourceHealth] = []
   disks: list[VsanPhysicalDiskHealth] = []
   componentsWithIssues: list[VsanResourceHealth] = []
   hostname: Optional[str] = None
   hostDedupScope: Optional[int] = None
   error: Optional[MethodFault] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import NoReturn
from typing import Optional

from pyVmomi.VmomiSupport import Enum
from pyVmomi.VmomiSupport import ManagedObject

from pyVmomi.vim import Description
from pyVmomi.vim import HostSystem
from pyVmomi.vim import Task

from pyVmomi.vmodl import DynamicData

class DiagnosticManager(ManagedObject):
   class LogDescriptor(DynamicData):
      class Creator(Enum):
         vpxd: ClassVar['Creator'] = 'vpxd'
         vpxa: ClassVar['Creator'] = 'vpxa'
         hostd: ClassVar['Creator'] = 'hostd'
         serverd: ClassVar['Creator'] = 'serverd'
         install: ClassVar['Creator'] = 'install'
         vpxClient: ClassVar['Creator'] = 'vpxClient'
         recordLog: ClassVar['Creator'] = 'recordLog'

      class Format(Enum):
         plain: ClassVar['Format'] = 'plain'

      key: str
      fileName: str
      creator: str
      format: str
      mimeType: str
      info: Description

   class LogHeader(DynamicData):
      lineStart: int
      lineEnd: int
      lineText: list[str] = []

   class BundleInfo(DynamicData):
      system: Optional[HostSystem] = None
      url: str

   class AuditRecordResult(DynamicData):
      records: list[str] = []
      nextToken: str

   def QueryDescriptions(self, host: Optional[HostSystem]) -> list[LogDescriptor]: ...
   def Browse(self, host: Optional[HostSystem], key: str, start: Optional[int], lines: Optional[int]) -> LogHeader: ...
   def GenerateLogBundles(self, includeDefault: bool, host: list[HostSystem]) -> Task: ...
   def FetchAuditRecords(self, token: Optional[str]) -> AuditRecordResult: ...
   def EmitSyslogMark(self, message: str) -> NoReturn: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vim.fault import VmConfigFault

class QuarantineModeFault(VmConfigFault):
   class FaultType(Enum):
      NoCompatibleNonQuarantinedHost: ClassVar['FaultType'] = 'NoCompatibleNonQuarantinedHost'
      CorrectionDisallowed: ClassVar['FaultType'] = 'CorrectionDisallowed'
      CorrectionImpact: ClassVar['FaultType'] = 'CorrectionImpact'

   vmName: str
   faultType: str



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.VmomiSupport import ManagedObject

from pyVmomi.vim import ClusterComputeResource
from pyVmomi.vim import Task

from pyVmomi.vim.vsan import ResourceCheckSpec
from pyVmomi.vim.vsan import ResourceCheckStatus

class VsanResourceCheckSystem(ManagedObject):
   def PerformResourceCheck(self, resourceCheckSpec: ResourceCheckSpec, cluster: Optional[ClusterComputeResource]) -> Task: ...
   def GetResourceCheckStatus(self, resourceCheckSpec: Optional[ResourceCheckSpec], cluster: Optional[ClusterComputeResource]) -> ResourceCheckStatus: ...
   def PerformResourceCheckOnHost(self, resourceCheckSpec: ResourceCheckSpec) -> Task: ...
   def CancelResourceCheckOnHost(self) -> bool: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.vsan import DataEfficiencyConfig
from pyVmomi.vim.vsan import DataEncryptionConfig

from pyVmomi.vim.vsan.host import DiskMapping

class DiskMapInfoEx(DynamicData):
   mapping: DiskMapping
   isMounted: bool
   unlockedEncrypted: Optional[bool] = None
   isAllFlash: bool
   isDataEfficiency: Optional[bool] = None
   encryptionInfo: Optional[DataEncryptionConfig] = None
   dataEfficiencyConfig: Optional[DataEfficiencyConfig] = None
   diskgroupCapability: list[str] = []



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.host import NtpConfig
from pyVmomi.vim.host import PtpConfig

class DateTimeConfig(DynamicData):
   timeZone: Optional[str] = None
   ntpConfig: Optional[NtpConfig] = None
   ptpConfig: Optional[PtpConfig] = None
   protocol: Optional[str] = None
   enabled: Optional[bool] = None
   disableEvents: Optional[bool] = None
   disableFallback: Optional[bool] = None
   resetToFactoryDefaults: Optional[bool] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import NoReturn
from typing import Optional

from pyVmomi.VmomiSupport import ManagedObject
from pyVmomi.VmomiSupport import long

from pyVmomi.vim import ClusterComputeResource
from pyVmomi.vim import ComputeResource
from pyVmomi.vim import HostSystem
from pyVmomi.vim import Task

from pyVmomi.vim.cluster import VsanDiagnosticsThreshold
from pyVmomi.vim.cluster import VsanNetworkDiagnostics

from pyVmomi.vim.vm import ProfileSpec

from pyVmomi.vim.vsan import IODiagnosticsInstance
from pyVmomi.vim.vsan import IODiagnosticsInstanceQuerySpec
from pyVmomi.vim.vsan import IODiagnosticsTarget
from pyVmomi.vim.vsan import IODiagnosticsTargetStats
from pyVmomi.vim.vsan import VsanIOTripAnalyzerConfig
from pyVmomi.vim.vsan import VsanIOTripAnalyzerRecurrence

class VsanDiagnosticsSystem(ManagedObject):
   def QueryNetworkDiagnostics(self, cluster: ComputeResource, host: Optional[HostSystem]) -> list[VsanNetworkDiagnostics]: ...
   def GetThresholds(self, cluster: ComputeResource, entityType: Optional[str], metric: Optional[str]) -> list[VsanDiagnosticsThreshold]: ...
   def SetThresholds(self, cluster: ComputeResource, thresholds: list[VsanDiagnosticsThreshold]) -> NoReturn: ...
   def StartIODiagnosticsTask(self, targets: list[IODiagnosticsTarget], cluster: Optional[ClusterComputeResource], duration: Optional[long]) -> Task: ...
   def QueryIODiagnosticsInstances(self, querySpec: IODiagnosticsInstanceQuerySpec, cluster: Optional[ClusterComputeResource]) -> list[IODiagnosticsInstance]: ...
   def QueryIODiagnosticsStats(self, instanceName: str, cluster: Optional[ClusterComputeResource]) -> list[IODiagnosticsTargetStats]: ...
   def CreateIOTripAnalyzerRecurrences(self, cluster: ComputeResource, recurrences: list[VsanIOTripAnalyzerRecurrence]) -> list[VsanIOTripAnalyzerRecurrence]: ...
   def GetIOTripAnalyzerSchedulerConfig(self, cluster: ComputeResource) -> Optional[VsanIOTripAnalyzerConfig]: ...
   def EditIOTripAnalyzerRecurrences(self, cluster: ComputeResource, recurrences: list[VsanIOTripAnalyzerRecurrence]) -> list[VsanIOTripAnalyzerRecurrence]: ...
   def RemoveIOTripAnalyzerRecurrences(self, cluster: ComputeResource, names: list[str]) -> NoReturn: ...
   def SetTraceObjectPolicy(self, cluster: Optional[ComputeResource], traceObjectUuid: str, profile: Optional[ProfileSpec]) -> bool: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import NoReturn
from typing import Optional

from pyVmomi.VmomiSupport import ManagedObject
from pyVmomi.VmomiSupport import long

from pyVmomi.vim import ClusterComputeResource
from pyVmomi.vim import HostSystem
from pyVmomi.vim import Task
from pyVmomi.vim import VirtualMachine

from pyVmomi.vim.cluster import VsanIoInsightInstance
from pyVmomi.vim.cluster import VsanIoInsightInstanceQuerySpec

from pyVmomi.vim.host import VsanHostIoInsightInfo

class VsanIoInsightManager(ManagedObject):
   def StartIoInsight(self, cluster: Optional[ClusterComputeResource], runName: Optional[str], durationSec: Optional[long], targetHosts: list[HostSystem], targetVMs: list[VirtualMachine]) -> Task: ...
   def StopIoInsight(self, cluster: Optional[ClusterComputeResource], runName: Optional[str], hostsIoInsightInfos: list[VsanHostIoInsightInfo]) -> Task: ...
   def QueryIoInsightInstances(self, querySpec: VsanIoInsightInstanceQuerySpec, cluster: Optional[ClusterComputeResource]) -> list[VsanIoInsightInstance]: ...
   def DeleteIoInsightInstance(self, runName: str, cluster: Optional[ClusterComputeResource]) -> NoReturn: ...
   def RenameIoInsightInstance(self, oldRunName: str, newRunName: str, cluster: Optional[ClusterComputeResource]) -> NoReturn: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import Optional

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vim.vm.device import VirtualController

class VirtualSCSIController(VirtualController):
   class Sharing(Enum):
      noSharing: ClassVar['Sharing'] = 'noSharing'
      virtualSharing: ClassVar['Sharing'] = 'virtualSharing'
      physicalSharing: ClassVar['Sharing'] = 'physicalSharing'

   hotAddRemove: Optional[bool] = None
   sharedBus: Sharing
   scsiCtlrUnitNumber: Optional[int] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.VmomiSupport import long

from pyVmomi.vim.cns import BackingObjectDetails

class BlockBackingDetails(BackingObjectDetails):
   backingDiskId: Optional[str] = None
   backingDiskUrlPath: Optional[str] = None
   backingDiskPath: Optional[str] = None
   backingDiskObjectId: Optional[str] = None
   usedCapacityInMb: Optional[long] = None
   aggregatedSnapshotCapacityInMb: Optional[long] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vim import Datastore
from pyVmomi.vim import LicenseManager
from pyVmomi.vim import Network

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.host import Capability
from pyVmomi.vim.host import Summary

from pyVmomi.vim.vm import Summary

class ConnectInfo(DynamicData):
   class NetworkInfo(DynamicData):
      summary: Network.Summary

   class NewNetworkInfo(NetworkInfo):
      pass

   class DatastoreInfo(DynamicData):
      summary: Datastore.Summary

   class DatastoreExistsInfo(DatastoreInfo):
      newDatastoreName: str

   class DatastoreNameConflictInfo(DatastoreInfo):
      newDatastoreName: str

   class LicenseInfo(DynamicData):
      license: LicenseManager.LicenseInfo
      evaluation: LicenseManager.EvaluationInfo
      resource: Optional[LicenseManager.LicensableResourceInfo] = None

   serverIp: Optional[str] = None
   inDasCluster: Optional[bool] = None
   host: Summary
   vm: list[Summary] = []
   vimAccountNameRequired: Optional[bool] = None
   clusterSupported: Optional[bool] = None
   network: list[NetworkInfo] = []
   datastore: list[DatastoreInfo] = []
   license: Optional[LicenseInfo] = None
   capability: Optional[Capability] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from . import fault as fault
from . import issue as issue
from . import vib as vib

from .Agency import Agency as Agency
from .Agent import Agent as Agent
from .EamObject import EamObject as EamObject
from .EsxAgentManager import EsxAgentManager as EsxAgentManager
from .Task import Task as Task



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from datetime import datetime

from typing import Optional

from pyVmomi.vim import HostSystem

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.profile.host import HostSubSpecification

class HostSpecification(DynamicData):
   createdTime: datetime
   lastModified: Optional[datetime] = None
   host: HostSystem
   subSpecs: list[HostSubSpecification] = []
   changeID: Optional[str] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import Optional

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vmodl import DynamicData
from pyVmomi.vmodl import MethodFault

from pyVmomi.vim.encryption import CryptoKeyId

class ToolsConfigInfo(DynamicData):
   class UpgradePolicy(Enum):
      manual: ClassVar['UpgradePolicy'] = 'manual'
      upgradeAtPowerCycle: ClassVar['UpgradePolicy'] = 'upgradeAtPowerCycle'

   class ToolsLastInstallInfo(DynamicData):
      counter: int
      fault: Optional[MethodFault] = None

   toolsVersion: Optional[int] = None
   toolsInstallType: Optional[str] = None
   afterPowerOn: Optional[bool] = None
   afterResume: Optional[bool] = None
   beforeGuestStandby: Optional[bool] = None
   beforeGuestShutdown: Optional[bool] = None
   beforeGuestReboot: Optional[bool] = None
   toolsUpgradePolicy: Optional[str] = None
   pendingCustomization: Optional[str] = None
   customizationKeyId: Optional[CryptoKeyId] = None
   syncTimeWithHostAllowed: Optional[bool] = None
   syncTimeWithHost: Optional[bool] = None
   lastInstallInfo: Optional[ToolsLastInstallInfo] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from datetime import datetime

from typing import Optional

from pyVmomi.vim import ManagedEntity

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.alarm import Alarm

class AlarmState(DynamicData):
   key: str
   entity: ManagedEntity
   alarm: Alarm
   overallStatus: ManagedEntity.Status
   time: datetime
   acknowledged: Optional[bool] = None
   acknowledgedByUser: Optional[str] = None
   acknowledgedTime: Optional[datetime] = None
   eventKey: Optional[int] = None
   disabled: Optional[bool] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import NoReturn
from typing import Optional

from pyVmomi.VmomiSupport import ManagedObject

from pyVmomi.pbm import ServerObjectRef

from pyVmomi.pbm.placement import PlacementHub

from pyVmomi.pbm.profile import CapabilityBasedProfileCreateSpec
from pyVmomi.pbm.profile import CapabilityBasedProfileUpdateSpec
from pyVmomi.pbm.profile import DefaultProfileInfo
from pyVmomi.pbm.profile import Profile
from pyVmomi.pbm.profile import ProfileId
from pyVmomi.pbm.profile import ProfileOperationOutcome
from pyVmomi.pbm.profile import QueryProfileResult
from pyVmomi.pbm.profile import ResourceType

from pyVmomi.pbm.capability.provider import CapabilityObjectMetadataPerCategory
from pyVmomi.pbm.capability.provider import CapabilityObjectSchema

from pyVmomi.pbm.profile.provider import DatastoreSpaceStatistics

class ProfileManager(ManagedObject):
   def FetchResourceType(self) -> list[ResourceType]: ...
   def FetchVendorInfo(self, resourceType: Optional[ResourceType]) -> list[CapabilityObjectSchema.VendorResourceTypeInfo]: ...
   def FetchCapabilityMetadata(self, resourceType: Optional[ResourceType], vendorUuid: Optional[str]) -> list[CapabilityObjectMetadataPerCategory]: ...
   def FetchCapabilitySchema(self, vendorUuid: Optional[str], lineOfService: list[str]) -> list[CapabilityObjectSchema]: ...
   def Create(self, createSpec: CapabilityBasedProfileCreateSpec) -> ProfileId: ...
   def Update(self, profileId: ProfileId, updateSpec: CapabilityBasedProfileUpdateSpec) -> NoReturn: ...
   def Delete(self, profileId: list[ProfileId]) -> list[ProfileOperationOutcome]: ...
   def QueryProfile(self, resourceType: ResourceType, profileCategory: Optional[str]) -> list[ProfileId]: ...
   def RetrieveContent(self, profileIds: list[ProfileId]) -> list[Profile]: ...
   def QueryAssociatedProfiles(self, entities: list[ServerObjectRef]) -> list[QueryProfileResult]: ...
   def QueryAssociatedProfile(self, entity: ServerObjectRef) -> list[ProfileId]: ...
   def QueryAssociatedEntity(self, profile: ProfileId, entityType: Optional[str]) -> list[ServerObjectRef]: ...
   def QueryDefaultRequirementProfile(self, hub: PlacementHub) -> Optional[ProfileId]: ...
   def ResetDefaultRequirementProfile(self, profile: Optional[ProfileId]) -> NoReturn: ...
   def AssignDefaultRequirementProfile(self, profile: ProfileId, datastores: list[PlacementHub]) -> NoReturn: ...
   def FindApplicableDefaultProfile(self, datastores: list[PlacementHub]) -> list[Profile]: ...
   def QueryDefaultRequirementProfiles(self, datastores: list[PlacementHub]) -> list[DefaultProfileInfo]: ...
   def ResetVSanDefaultProfile(self) -> NoReturn: ...
   def QuerySpaceStatsForStorageContainer(self, datastore: ServerObjectRef, capabilityProfileId: list[ProfileId]) -> list[DatastoreSpaceStatistics]: ...
   def QueryAssociatedEntities(self, profiles: list[ProfileId]) -> list[QueryProfileResult]: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.VmomiSupport import long

from pyVmomi.vmodl import DynamicData

class EntityResourceCheckDetails(DynamicData):
   name: Optional[str] = None
   uuid: Optional[str] = None
   isNew: Optional[bool] = None
   capacity: Optional[long] = None
   postOperationCapacity: Optional[long] = None
   usedCapacity: Optional[long] = None
   postOperationUsedCapacity: Optional[long] = None
   additionalRequiredCapacity: Optional[long] = None
   maxComponents: Optional[long] = None
   components: Optional[long] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vim.cns import QueryFilter

class KubernetesQueryFilter(QueryFilter):
   namespaces: list[str] = []
   podNames: list[str] = []
   pvcNames: list[str] = []
   pvNames: list[str] = []



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import NoReturn
from typing import Optional

from pyVmomi.vim import ExtensibleManagedObject

from pyVmomi.vim.host import PciPassthruConfig
from pyVmomi.vim.host import PciPassthruInfo
from pyVmomi.vim.host import SriovDevicePoolInfo

class PciPassthruSystem(ExtensibleManagedObject):
   @property
   def pciPassthruInfo(self) -> list[PciPassthruInfo]: ...
   @property
   def sriovDevicePoolInfo(self) -> list[SriovDevicePoolInfo]: ...

   def Refresh(self) -> NoReturn: ...
   def UpdatePassthruConfig(self, config: list[PciPassthruConfig]) -> NoReturn: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import Optional

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vmodl import DynamicData

class DefaultPowerOpInfo(DynamicData):
   class PowerOpType(Enum):
      soft: ClassVar['PowerOpType'] = 'soft'
      hard: ClassVar['PowerOpType'] = 'hard'
      preset: ClassVar['PowerOpType'] = 'preset'

   class StandbyActionType(Enum):
      checkpoint: ClassVar['StandbyActionType'] = 'checkpoint'
      powerOnSuspend: ClassVar['StandbyActionType'] = 'powerOnSuspend'

   powerOffType: Optional[str] = None
   suspendType: Optional[str] = None
   resetType: Optional[str] = None
   defaultPowerOffType: Optional[str] = None
   defaultSuspendType: Optional[str] = None
   defaultResetType: Optional[str] = None
   standbyAction: Optional[str] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from datetime import datetime

from typing import Optional

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.host import VsanHostHclInfo

from pyVmomi.vim.vsan import VsanUpdateItem

class VsanClusterHclInfo(DynamicData):
   hclDbLastUpdate: Optional[datetime] = None
   hclDbAgeHealth: Optional[str] = None
   hostResults: list[VsanHostHclInfo] = []
   updateItems: list[VsanUpdateItem] = []
   hclDbAbsent: Optional[bool] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import Optional

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vim import ManagedEntity

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.alarm import AlarmExpression

class EventAlarmExpression(AlarmExpression):
   class ComparisonOperator(Enum):
      equals: ClassVar['ComparisonOperator'] = 'equals'
      notEqualTo: ClassVar['ComparisonOperator'] = 'notEqualTo'
      startsWith: ClassVar['ComparisonOperator'] = 'startsWith'
      doesNotStartWith: ClassVar['ComparisonOperator'] = 'doesNotStartWith'
      endsWith: ClassVar['ComparisonOperator'] = 'endsWith'
      doesNotEndWith: ClassVar['ComparisonOperator'] = 'doesNotEndWith'

   class Comparison(DynamicData):
      attributeName: str
      operator: str
      value: str

   comparisons: list[Comparison] = []
   eventType: type
   eventTypeId: Optional[str] = None
   objectType: Optional[type] = None
   status: Optional[ManagedEntity.Status] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vim import ManagedEntity

from pyVmomi.vim.view import ManagedObjectView

class ContainerView(ManagedObjectView):
   @property
   def container(self) -> ManagedEntity: ...
   @property
   def type(self) -> list[type]: ...
   @property
   def recursive(self) -> bool: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.VmomiSupport import short

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.option import OptionValue

from pyVmomi.vim.vm.device import VirtualDevice

class VirtualPCIPassthrough(VirtualDevice):
   class DeviceBackingInfo(VirtualDevice.DeviceBackingInfo):
      id: str
      deviceId: str
      systemId: str
      vendorId: short

   class AllowedDevice(DynamicData):
      vendorId: int
      deviceId: int
      subVendorId: Optional[int] = None
      subDeviceId: Optional[int] = None
      revisionId: Optional[short] = None

   class DynamicBackingInfo(VirtualDevice.DeviceBackingInfo):
      allowedDevice: list[AllowedDevice] = []
      customLabel: Optional[str] = None
      assignedId: Optional[str] = None

   class PluginBackingInfo(VirtualDevice.BackingInfo):
      pass

   class VmiopBackingInfo(PluginBackingInfo):
      vgpu: Optional[str] = None
      vgpuMigrateDataSizeMB: Optional[int] = None
      migrateSupported: Optional[bool] = None
      enhancedMigrateCapability: Optional[bool] = None

   class DvxBackingInfo(VirtualDevice.BackingInfo):
      deviceClass: Optional[str] = None
      configParams: list[OptionValue] = []



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vmodl import DynamicData

class VsanBasicDeviceInfo(DynamicData):
   deviceName: str
   pciId: Optional[str] = None
   fwVersion: Optional[str] = None
   features: list[str] = []



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.VmomiSupport import long

from pyVmomi.vim import KeyValue

from pyVmomi.vmodl import DynamicData
from pyVmomi.vmodl import MethodFault

from pyVmomi.vim.host import VsanHclDiskInfo
from pyVmomi.vim.host import VsanHostCimProviderInfo
from pyVmomi.vim.host import VsanVcgDeviceInfo

from pyVmomi.vim.vsan import VsanHclDriverInfo

class VsanHclControllerInfo(DynamicData):
   deviceName: str
   deviceDisplayName: Optional[str] = None
   driverName: Optional[str] = None
   driverVersion: Optional[str] = None
   vendorId: Optional[long] = None
   deviceId: Optional[long] = None
   subVendorId: Optional[long] = None
   subDeviceId: Optional[long] = None
   extraInfo: list[KeyValue] = []
   deviceOnHcl: Optional[bool] = None
   releaseSupported: Optional[bool] = None
   releasesOnHcl: list[str] = []
   driverVersionsOnHcl: list[str] = []
   driverVersionSupported: Optional[bool] = None
   fwVersionSupported: Optional[bool] = None
   fwVersionOnHcl: list[str] = []
   cacheConfigSupported: Optional[bool] = None
   cacheConfigOnHcl: list[str] = []
   raidConfigSupported: Optional[bool] = None
   raidConfigOnHcl: list[str] = []
   fwVersion: Optional[str] = None
   raidConfig: Optional[str] = None
   cacheConfig: Optional[str] = None
   cimProviderInfo: Optional[VsanHostCimProviderInfo] = None
   usedByVsan: Optional[bool] = None
   disks: list[VsanHclDiskInfo] = []
   issues: list[MethodFault] = []
   remediableIssues: list[str] = []
   driversOnHcl: list[VsanHclDriverInfo] = []
   fwAuxVersion: Optional[str] = None
   queueDepth: Optional[long] = None
   queueDepthOnHcl: Optional[long] = None
   queueDepthSupported: Optional[bool] = None
   diskMode: Optional[str] = None
   diskModeOnHcl: list[str] = []
   diskModeSupported: Optional[bool] = None
   toolName: Optional[str] = None
   toolVersion: Optional[str] = None
   productId: Optional[str] = None
   diskCapacity: Optional[long] = None
   vcgEntryInfo: list[VsanVcgDeviceInfo] = []
   controllerType: Optional[str] = None
   userSelectedVcgId: Optional[int] = None
   vsanCompatibility: list[str] = []



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from datetime import datetime

from typing import ClassVar
from typing import Optional

from pyVmomi.VmomiSupport import Enum
from pyVmomi.VmomiSupport import ManagedObject

from pyVmomi.vim import Capability
from pyVmomi.vim import HostSystem
from pyVmomi.vim import ResourcePool
from pyVmomi.vim import ServiceInstanceContent
from pyVmomi.vim import VirtualMachine

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.event import Event

class ServiceInstance(ManagedObject):
   class ValidateMigrationTestType(Enum):
      sourceTests: ClassVar['ValidateMigrationTestType'] = 'sourceTests'
      compatibilityTests: ClassVar['ValidateMigrationTestType'] = 'compatibilityTests'
      diskAccessibilityTests: ClassVar['ValidateMigrationTestType'] = 'diskAccessibilityTests'
      resourceTests: ClassVar['ValidateMigrationTestType'] = 'resourceTests'

   class VMotionCompatibilityType(Enum):
      cpu: ClassVar['VMotionCompatibilityType'] = 'cpu'
      software: ClassVar['VMotionCompatibilityType'] = 'software'

   class HostVMotionCompatibility(DynamicData):
      host: HostSystem
      compatibility: list[str] = []

   class ProductComponentInfo(DynamicData):
      id: str
      name: str
      version: str
      release: int

   @property
   def serverClock(self) -> datetime: ...
   @property
   def capability(self) -> Capability: ...
   @property
   def content(self) -> ServiceInstanceContent: ...

   def CurrentTime(self) -> datetime: ...
   def RetrieveContent(self) -> ServiceInstanceContent: ...
   def ValidateMigration(self, vm: list[VirtualMachine], state: Optional[VirtualMachine.PowerState], testType: list[str], pool: Optional[ResourcePool], host: Optional[HostSystem]) -> list[Event]: ...
   def QueryVMotionCompatibility(self, vm: VirtualMachine, host: list[HostSystem], compatibility: list[str]) -> list[HostVMotionCompatibility]: ...
   def RetrieveProductComponents(self) -> list[ProductComponentInfo]: ...



# Copyright (c) 2006-2024 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from . import fault as fault

from .DataObject import DataObject as DataObject
from .DynamicArray import DynamicArray as DynamicArray
from .DynamicData import DynamicData as DynamicData
from .DynamicProperty import DynamicProperty as DynamicProperty
from .KeyAnyValue import KeyAnyValue as KeyAnyValue
from .LocalizableMessage import LocalizableMessage as LocalizableMessage
from .LocalizedMethodFault import LocalizedMethodFault as LocalizedMethodFault
from .ManagedObject import ManagedObject as ManagedObject
from .MethodFault import MethodFault as MethodFault
from .RuntimeFault import RuntimeFault as RuntimeFault



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import NoReturn

from pyVmomi.VmomiSupport import ManagedObject

from pyVmomi.vim import HostSystem
from pyVmomi.vim import Task

from pyVmomi.vim.cluster import VsanVumSystemConfig

class VsanVumSystem(ManagedObject):
   def GetVsanVumConfig(self) -> VsanVumSystemConfig: ...
   def FetchIsoDepotCookie(self, username: str, password: str) -> NoReturn: ...
   def UpdateHostFirmware(self, host: HostSystem) -> Task: ...
   def UploadReleaseDb(self, db: str) -> NoReturn: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from datetime import datetime

from typing import Optional

from pyVmomi.VmomiSupport import long

from pyVmomi.vim import HostSystem
from pyVmomi.vim import VirtualMachine

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.host import FeatureMask

from pyVmomi.vim.vm import DeviceRuntimeInfo
from pyVmomi.vim.vm import FeatureRequirement
from pyVmomi.vim.vm import QuestionInfo

class RuntimeInfo(DynamicData):
   class DasProtectionState(DynamicData):
      dasProtected: bool

   device: list[DeviceRuntimeInfo] = []
   host: Optional[HostSystem] = None
   connectionState: VirtualMachine.ConnectionState
   powerState: VirtualMachine.PowerState
   vmFailoverInProgress: Optional[bool] = None
   faultToleranceState: VirtualMachine.FaultToleranceState
   dasVmProtection: Optional[DasProtectionState] = None
   toolsInstallerMounted: bool
   suspendTime: Optional[datetime] = None
   bootTime: Optional[datetime] = None
   suspendInterval: Optional[long] = None
   question: Optional[QuestionInfo] = None
   memoryOverhead: Optional[long] = None
   maxCpuUsage: Optional[int] = None
   maxMemoryUsage: Optional[int] = None
   numMksConnections: int
   recordReplayState: VirtualMachine.RecordReplayState
   cleanPowerOff: Optional[bool] = None
   needSecondaryReason: Optional[str] = None
   onlineStandby: bool
   minRequiredEVCModeKey: Optional[str] = None
   consolidationNeeded: bool
   offlineFeatureRequirement: list[FeatureRequirement] = []
   featureRequirement: list[FeatureRequirement] = []
   featureMask: list[FeatureMask] = []
   vFlashCacheAllocation: Optional[long] = None
   paused: Optional[bool] = None
   snapshotInBackground: Optional[bool] = None
   quiescedForkParent: Optional[bool] = None
   instantCloneFrozen: Optional[bool] = None
   cryptoState: Optional[str] = None
   suspendedToMemory: Optional[bool] = None
   opNotificationTimeout: Optional[long] = None
   iommuActive: Optional[bool] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vmodl import DynamicData

class TdxInfo(DynamicData):
   class TdxState(Enum):
      initializing: ClassVar['TdxState'] = 'initializing'
      initialized: ClassVar['TdxState'] = 'initialized'
      configured: ClassVar['TdxState'] = 'configured'
      ready: ClassVar['TdxState'] = 'ready'

   tdxState: str
   numTDXPrivateKeyIDs: int



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vmodl import DynamicData

class UsbScanCodeSpec(DynamicData):
   class ModifierType(DynamicData):
      leftControl: Optional[bool] = None
      leftShift: Optional[bool] = None
      leftAlt: Optional[bool] = None
      leftGui: Optional[bool] = None
      rightControl: Optional[bool] = None
      rightShift: Optional[bool] = None
      rightAlt: Optional[bool] = None
      rightGui: Optional[bool] = None

   class KeyEvent(DynamicData):
      usbHidCode: int
      modifiers: Optional[ModifierType] = None

   keyEvents: list[KeyEvent] = []



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from . import replication as replication

from .AuthConnectionFailed import AuthConnectionFailed as AuthConnectionFailed
from .CertificateAuthorityFault import CertificateAuthorityFault as CertificateAuthorityFault
from .CertificateNotImported import CertificateNotImported as CertificateNotImported
from .CertificateNotTrusted import CertificateNotTrusted as CertificateNotTrusted
from .CertificateRefreshFailed import CertificateRefreshFailed as CertificateRefreshFailed
from .CertificateRevocationFailed import CertificateRevocationFailed as CertificateRevocationFailed
from .DuplicateEntry import DuplicateEntry as DuplicateEntry
from .InactiveProvider import InactiveProvider as InactiveProvider
from .IncorrectUsernamePassword import IncorrectUsernamePassword as IncorrectUsernamePassword
from .InvalidCertificate import InvalidCertificate as InvalidCertificate
from .InvalidLogin import InvalidLogin as InvalidLogin
from .InvalidProfile import InvalidProfile as InvalidProfile
from .InvalidSession import InvalidSession as InvalidSession
from .InvalidUrl import InvalidUrl as InvalidUrl
from .MultipleSortSpecsNotSupported import MultipleSortSpecsNotSupported as MultipleSortSpecsNotSupported
from .NoCommonProviderForAllBackings import NoCommonProviderForAllBackings as NoCommonProviderForAllBackings
from .NotSupportedByProvider import NotSupportedByProvider as NotSupportedByProvider
from .ProviderBusy import ProviderBusy as ProviderBusy
from .ProviderConnectionFailed import ProviderConnectionFailed as ProviderConnectionFailed
from .ProviderNotFound import ProviderNotFound as ProviderNotFound
from .ProviderOutOfProvisioningResource import ProviderOutOfProvisioningResource as ProviderOutOfProvisioningResource
from .ProviderOutOfResource import ProviderOutOfResource as ProviderOutOfResource
from .ProviderRegistrationFault import ProviderRegistrationFault as ProviderRegistrationFault
from .ProviderSyncFailed import ProviderSyncFailed as ProviderSyncFailed
from .ProviderUnavailable import ProviderUnavailable as ProviderUnavailable
from .ProviderUnregistrationFault import ProviderUnregistrationFault as ProviderUnregistrationFault
from .ProxyRegistrationFailed import ProxyRegistrationFailed as ProxyRegistrationFailed
from .QueryExecutionFault import QueryExecutionFault as QueryExecutionFault
from .QueryNotSupported import QueryNotSupported as QueryNotSupported
from .ResourceInUse import ResourceInUse as ResourceInUse
from .ServiceNotInitialized import ServiceNotInitialized as ServiceNotInitialized
from .SmsFault import SmsFault as SmsFault
from .SyncInProgress import SyncInProgress as SyncInProgress
from .TooMany import TooMany as TooMany



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import NoReturn
from typing import Optional

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vim import Task

from pyVmomi.vim.host import DirectoryStore

class ActiveDirectoryAuthentication(DirectoryStore):
   class CertificateDigest(Enum):
      SHA1: ClassVar['CertificateDigest'] = 'SHA1'

   def JoinDomain(self, domainName: str, userName: str, password: str) -> Task: ...
   def JoinDomainWithCAM(self, domainName: str, camServer: str) -> Task: ...
   def ImportCertificateForCAM(self, certPath: str, camServer: str) -> Task: ...
   def LeaveCurrentDomain(self, force: bool) -> Task: ...
   def EnableSmartCardAuthentication(self) -> NoReturn: ...
   def InstallSmartCardTrustAnchor(self, cert: str) -> NoReturn: ...
   def ReplaceSmartCardTrustAnchors(self, certs: list[str]) -> NoReturn: ...
   def RemoveSmartCardTrustAnchor(self, issuer: str, serial: str) -> NoReturn: ...
   def RemoveSmartCardTrustAnchorByFingerprint(self, fingerprint: str, digest: str) -> NoReturn: ...
   def RemoveSmartCardTrustAnchorCertificate(self, certificate: str) -> NoReturn: ...
   def ListSmartCardTrustAnchors(self) -> list[str]: ...
   def DisableSmartCardAuthentication(self) -> NoReturn: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.VmomiSupport import ManagedObject

from pyVmomi.vim import Datacenter
from pyVmomi.vim import ManagedEntity
from pyVmomi.vim import VirtualMachine

class SearchIndex(ManagedObject):
   def FindByUuid(self, datacenter: Optional[Datacenter], uuid: str, vmSearch: bool, instanceUuid: Optional[bool]) -> Optional[ManagedEntity]: ...
   def FindByDatastorePath(self, datacenter: Datacenter, path: str) -> Optional[VirtualMachine]: ...
   def FindByDnsName(self, datacenter: Optional[Datacenter], dnsName: str, vmSearch: bool) -> Optional[ManagedEntity]: ...
   def FindByIp(self, datacenter: Optional[Datacenter], ip: str, vmSearch: bool) -> Optional[ManagedEntity]: ...
   def FindByInventoryPath(self, inventoryPath: str) -> Optional[ManagedEntity]: ...
   def FindChild(self, entity: ManagedEntity, name: str) -> Optional[ManagedEntity]: ...
   def FindAllByUuid(self, datacenter: Optional[Datacenter], uuid: str, vmSearch: bool, instanceUuid: Optional[bool]) -> list[ManagedEntity]: ...
   def FindAllByDnsName(self, datacenter: Optional[Datacenter], dnsName: str, vmSearch: bool) -> list[ManagedEntity]: ...
   def FindAllByIp(self, datacenter: Optional[Datacenter], ip: str, vmSearch: bool) -> list[ManagedEntity]: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from .AgentIssue import AgentIssue as AgentIssue
from .CertificateNotTrusted import CertificateNotTrusted as CertificateNotTrusted
from .HostInMaintenanceMode import HostInMaintenanceMode as HostInMaintenanceMode
from .HostInPartialMaintenanceMode import HostInPartialMaintenanceMode as HostInPartialMaintenanceMode
from .InsufficientClusterResources import InsufficientClusterResources as InsufficientClusterResources
from .InsufficientClusterSpace import InsufficientClusterSpace as InsufficientClusterSpace
from .InvalidConfig import InvalidConfig as InvalidConfig
from .MissingClusterVmDatastore import MissingClusterVmDatastore as MissingClusterVmDatastore
from .MissingClusterVmNetwork import MissingClusterVmNetwork as MissingClusterVmNetwork
from .OvfInvalidProperty import OvfInvalidProperty as OvfInvalidProperty
from .TransitionFailed import TransitionFailed as TransitionFailed
from .VmHookFailed import VmHookFailed as VmHookFailed
from .VmHookTimedout import VmHookTimedout as VmHookTimedout
from .VmInaccessible import VmInaccessible as VmInaccessible
from .VmIssue import VmIssue as VmIssue
from .VmNotDeployed import VmNotDeployed as VmNotDeployed
from .VmNotRemoved import VmNotRemoved as VmNotRemoved
from .VmPoweredOff import VmPoweredOff as VmPoweredOff
from .VmPoweredOn import VmPoweredOn as VmPoweredOn
from .VmProtected import VmProtected as VmProtected
from .VmSuspended import VmSuspended as VmSuspended



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.VmomiSupport import ManagedObject

from pyVmomi.vim import Task

from pyVmomi.vmodl import DynamicData
from pyVmomi.vmodl import LocalizableMessage

class ClusterPowerContext(DynamicData):
   currentClusterPowerStatus: str
   orderedClusterPowerStatus: list[str] = []
   trackingTask: Optional[Task] = None
   lastErrorMessage: Optional[LocalizableMessage] = None
   lastErrorMOs: list[ManagedObject] = []



#############################################################
# Copyright (c) 2012-2025 Broadcom. All Rights Reserved.
# The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.
# A python helper module to do SSO related operations.
#############################################################
__author__ = 'VMware, Inc.'

#Standard library imports.
import re
import datetime
import base64
import hashlib
from uuid import uuid4
from io import BytesIO
from pyVmomi.five import PY3, HTTPConnection, HTTPSConnection, urlparse
if PY3:
    from html import escape
else:
    from cgi import escape
#Third-party imports.
from lxml import etree
from OpenSSL import crypto
import ssl

from pyVmomi.Security import VerifyCertThumbprint

UTF_8 = 'utf-8'
SHA256 = 'sha256'
SHA512 = 'sha512'

# add default parser to etree with resolve_entities set to False
default_parser = etree.XMLParser(resolve_entities=False)
etree.set_default_parser(default_parser)

def _extract_certificate(cert):
    """
    Extract DER certificate/private key from DER/base64-ed DER/PEM string.

    @type           cert: C{str}
    @param          cert: Certificate/private key in one of three supported formats.

    @rtype: C{str}
    @return: Certificate/private key in DER (binary ASN.1) format.
    """
    if not cert:
        raise IOError('Empty certificate')
    signature = cert[0]
    # DER certificate is sequence.  ASN.1 sequence is 0x30.
    if signature == '\x30':
        return cert
    # PEM without preamble.  Base64-encoded 0x30 is 0x4D.
    if signature == '\x4D':
        return base64.b64decode(cert)
    # PEM with preamble.  Starts with '-'.
    if signature == '-':
        return base64.b64decode(re.sub('-----[A-Z ]*-----', '', cert))
    # Unknown format.
    raise IOError('Invalid certificate file format')


#time.strftime() method returns 6 digit millisecond
#Formatting time with 3 digit milliseconds
def format_time(time):
    return time[:-3] + 'Z'


class SoapException(Exception):
    """
    Exception raised in case of STS request failure.
    """
    def __init__(self, soap_msg, fault_code, fault_string):
        """
        Initializer for SoapException.

        @type      soap_msg: C{str}
        @param     soap_msg: the soap fault XML returned by STS
        @type    fault_code: C{str}
        @param   fault_code: The fault code returned by STS.
        @type  fault_string: C{str}
        @param fault_string: The fault string returned by STS.
        """
        self._soap_msg = soap_msg
        self._fault_code = fault_code
        self._fault_string = fault_string
        Exception.__init__(self)

    def __str__(self):
        """
        Returns the string representation of SoapException.

        @rtype: C{str}
        @return: string representation of SoapException
        """
        return ("SoapException:\nfaultcode: %(_fault_code)s\n"
                "faultstring: %(_fault_string)s\n"
                "faultxml: %(_soap_msg)s" % self.__dict__)


class _SSOHTTPSConnection(HTTPSConnection):
    """
    An HTTPS class that verifies server's certificate on connect.
    """
    def __init__(self, host, context, thumbprint=None, server_cert=None):
        """
        Initializer.  See HTTPSConnection for other arguments
        than thumbprint and server_cert.

        At least one of thumbprint, server_cert should be provided,
        otherwise server certificate is not validated.

        @type           thumbprint: C(str)
        @param          thumbprint: Expected SHA-1 thumbprint of the server
                                    certificate.  May be None.

        @type          server_cert: C(str)
        @param         server_cert: File with expected server certificate.
                                    May be None.
        """
        self.server_thumbprint = thumbprint
        if self.server_thumbprint is not None:
            self.server_thumbprint = re.sub(':', '',
                                            self.server_thumbprint.lower())
        server_cert_path = server_cert
        if server_cert_path is not None:
            with open(server_cert_path, 'rb') as f:
                server_cert = f.read().decode(UTF_8)
            self.server_cert = _extract_certificate(server_cert)
        else:
            self.server_cert = None
        HTTPSConnection.__init__(self, host=host, context=context)

    def _check_cert(self, peerCert):
        """
        Verify that peer certificate matches one we expect.

        @type             peerCert: C(str)
        @param            peerCert: Server certificate in DER format.

        @rtype: boolean
        @return: True if peerCert is acceptable.  False otherwise.
        """
        try:
            if self.server_cert is not None:
                if peerCert != self.server_cert:
                    raise Exception("Invalid certificate")
            if self.server_thumbprint:
                VerifyCertThumbprint(peerCert, self.server_thumbprint)
        except Exception:
            self.sock.close()
            self.sock = None
            raise

    def connect(self):
        """
        Connect method: connects to the remote system, and upon
        successful connection validates certificate.

        Throws an exception when something is wrong.  See
        HTTPSConnection.connect() for details.
        """
        HTTPSConnection.connect(self)

        self._check_cert(self.sock.getpeercert(True))


def is_sidecar_request(scheme, host):
    """
    Check if the request is through
    local sidecar (localhost:1080)
    """
    if scheme == "http" and host == "localhost:1080":
        return True


class SsoAuthenticator(object):
    """
    A class to handle the transport layer communication between the client and
    the STS service.
    """
    def __init__(self, sts_url, sts_cert=None, thumbprint=None):
        """
        Initializer for SsoAuthenticator.

        @type           sts_url: C{str}
        @param          sts_url: URL for the Security Token Service. Usually
                                 obtained by querying Component Manager.
        @type          sts_cert: C{str}
        @param         sts_cert: The file with public key of the Security
                                 Token Service.  Usually obtained from
                                 Component Manager and written to the file.
        @type        thumbprint: C{str}
        @param       thumbprint: The SHA-1 thumbprint of the certificate used
                                 by the Security Token Service.  It is same
                                 thumbprint you can pass to pyVmomi SoapAdapter.
        """
        self._sts_cert = sts_cert
        self._sts_url = sts_url
        self._sts_thumbprint = thumbprint

    def perform_request(self,
                        soap_message,
                        public_key=None,
                        private_key=None,
                        ssl_context=None):
        """
        Performs a Holder-of-Key SAML token request using the service user's
        certificates or a bearer token request using the user credentials.

        @type      soap_message: C{str}
        @param     soap_message: Authentication SOAP request.
        @type        public_key: C{str}
        @param       public_key: File containing the public key for the service
                                 user registered with SSO, in PEM format.
        @type       private_key: C{str}
        @param      private_key: File containing the private key for the service
                                 user registered with SSO, in PEM format.
        @type       ssl_context: C{ssl.SSLContext}
        @param      ssl_context: SSL context describing the various SSL options.
        @rtype: C{str}
        @return: Response received from the STS after the HoK request.
        """
        parsed = urlparse(self._sts_url)
        host = parsed.netloc  # pylint: disable=E1101
        scheme = parsed.scheme
        encoded_message = soap_message.encode(UTF_8)

        if ssl_context is not None:
            sslContext = ssl_context
        else:
            sslContext = ssl._create_default_https_context()
        if public_key and private_key:
            sslContext.load_cert_chain(public_key, private_key)

        """
        Allow creation of HTTPConnection, only for calls routing
        through local sidecar (localhost:1080)
        """
        if is_sidecar_request(scheme, host):
            webservice = HTTPConnection(host=host)
        else:
            webservice = _SSOHTTPSConnection(host=host,
                                            server_cert=self._sts_cert,
                                            thumbprint=self._sts_thumbprint,
                                            context=ssl_context)


        webservice.putrequest("POST", parsed.path, skip_host=True)  # pylint: disable=E1101
        webservice.putheader("Host", host)
        webservice.putheader("User-Agent", "VMware/pyVmomi")
        webservice.putheader("Accept", "text/xml, multipart/related")
        webservice.putheader("Content-type", "text/xml; charset=\"UTF-8\"")
        webservice.putheader("Content-length", "%d" % len(encoded_message))
        webservice.putheader("Connection", "keep-alive")
        webservice.putheader(
            "SOAPAction",
            "http://docs.oasis-open.org/ws-sx/ws-trust/200512/RST/Issue")
        webservice.endheaders()
        webservice.send(encoded_message)

        saml_response = webservice.getresponse()
        if saml_response.status != 200:
            faultraw = saml_response.read()
            # Hopefully it is utf-8 or us-ascii, not Apache error message in Shift-JIS.
            fault = faultraw.decode(UTF_8)
            # Best effort at figuring out a SOAP fault.
            if saml_response.status == 500 and fault and 'faultcode' in fault:
                fault_xml = etree.fromstring(faultraw)
                parsed_fault = fault_xml.xpath("//text()")
                if len(parsed_fault) == 2:
                    raise SoapException(fault, *parsed_fault)
            raise Exception("Got response %s: %s\n%s" %
                            (saml_response.status, saml_response.msg, fault))
        return saml_response.read()

    def get_bearer_saml_assertion(self,
                                  username,
                                  password,
                                  public_key=None,
                                  private_key=None,
                                  request_duration=60,
                                  token_duration=600,
                                  delegatable=False,
                                  renewable=False,
                                  ssl_context=None):
        """
        Extracts the assertion from the Bearer Token received from the Security
        Token Service.

        @type          username: C{str}
        @param         username: Username for the user for which bearer token
                                 needs to be requested.
        @type          password: C{str}
        @param         password: Password for the user for which bearer token
                                 needs to be requested.
        @type        public_key: C{str}
        @param       public_key: File containing the public key for the service
                                 user registered with SSO, in PEM format.
        @type       private_key: C{str}
        @param      private_key: File containing the private key for the service
                                 user registered with SSO, in PEM format.

        @type  request_duration: C{long}
        @param request_duration: The duration for which the request is valid. If
                                 the STS receives this request after this
                                 duration, it is assumed to have expired. The
                                 duration is in seconds and the default is 60s.
        @type    token_duration: C{long}
        @param   token_duration: The duration for which the SAML token is issued
                                 for. The duration is specified in seconds and
                                 the default is 600s.
        @type       delegatable: C{boolean}
        @param      delegatable: Whether the generated token is delegatable or not
                                 The default value is False
        @type       ssl_context: C{ssl.SSLContext}
        @param      ssl_context: SSL context describing the various SSL options.
        @rtype: C{str}
        @return: The SAML assertion in Unicode.
        """
        request = SecurityTokenRequest(username=username,
                                       password=password,
                                       public_key=public_key,
                                       private_key=private_key,
                                       request_duration=request_duration,
                                       token_duration=token_duration)
        soap_message = request.construct_bearer_token_request(
            delegatable=delegatable, renewable=renewable)
        bearer_token = self.perform_request(soap_message, public_key,
                                            private_key, ssl_context)
        return etree.tostring(_extract_element(
            etree.fromstring(bearer_token), 'Assertion',
            {'saml2': "urn:oasis:names:tc:SAML:2.0:assertion"}),
                              pretty_print=False).decode(UTF_8)

    def get_hok_saml_assertion(self,
                               public_key,
                               private_key,
                               request_duration=60,
                               token_duration=600,
                               act_as_token=None,
                               delegatable=False,
                               renewable=False,
                               ssl_context=None):
        """
        Extracts the assertion from the response received from the Security
        Token Service.

        @type        public_key: C{str}
        @param       public_key: File containing the public key for the service
                                 user registered with SSO, in PEM format.
        @type       private_key: C{str}
        @param      private_key: File containing the private key for the service
                                 user registered with SSO, in PEM format.
        @type  request_duration: C{long}
        @param request_duration: The duration for which the request is valid. If
                                 the STS receives this request after this
                                 duration, it is assumed to have expired. The
                                 duration is in seconds and the default is 60s.
        @type    token_duration: C{long}
        @param   token_duration: The duration for which the SAML token is issued
                                 for. The duration is specified in seconds and
                                 the default is 600s.
        @type      act_as_token: C{str}
        @param     act_as_token: Bearer/Hok token which is delegatable
        @type       delegatable: C{boolean}
        @param      delegatable: Whether the generated token is delegatable or not
        @type         renewable: C{boolean}
        @param        renewable: Whether the generated token is renewable or not
                                 The default value is False
        @type       ssl_context: C{ssl.SSLContext}
        @param      ssl_context: SSL context describing the various SSL options.
        @rtype: C{str}
        @return: The SAML assertion in Unicode.
        """
        request = SecurityTokenRequest(public_key=public_key,
                                       private_key=private_key,
                                       request_duration=request_duration,
                                       token_duration=token_duration)
        soap_message = request.construct_hok_request(delegatable=delegatable,
                                                     act_as_token=act_as_token,
                                                     renewable=renewable)
        hok_token = self.perform_request(soap_message, public_key, private_key,
                                         ssl_context)
        return etree.tostring(_extract_element(
            etree.fromstring(hok_token), 'Assertion',
            {'saml2': "urn:oasis:names:tc:SAML:2.0:assertion"}),
                              pretty_print=False).decode(UTF_8)

    def get_token_by_token(self,
                           hok_token,
                           private_key,
                           request_duration=60,
                           token_duration=600,
                           delegatable=False,
                           renewable=False,
                           ssl_context=None):
        """
        Get Hok token by Hok token.

        @type       hok_token:   C{str}
        @param      hok_token:   Hok token to be used to get another token
        @type       private_key: C{str}
        @param      private_key: File containing the private key for the service
                                 user registered with SSO, in PEM format.
        @type  request_duration: C{long}
        @param request_duration: The duration for which the request is valid. If
                                 the STS receives this request after this
                                 duration, it is assumed to have expired. The
                                 duration is in seconds and the default is 60s.
        @type    token_duration: C{long}
        @param   token_duration: The duration for which the SAML token is issued
                                 for. The duration is specified in seconds and
                                 the default is 600s.
        @type    delegatable: C{boolean}
        @param   delegatable: Whether the generated token is delegatable or not.
                              The default value is False.
        @type         renewable: C{boolean}
        @param        renewable: Whether the generated token is renewable or not
                                 The default value is False
        @type       ssl_context: C{ssl.SSLContext}
        @param      ssl_context: SSL context describing the various SSL options.
        @rtype: C{str}
        @return: The Hok SAML assertion in Unicode.
        """
        request = SecurityTokenRequest(private_key=private_key,
                                       request_duration=request_duration,
                                       token_duration=token_duration,
                                       hok_token=hok_token)
        soap_message = request.construct_hok_by_hok_request(
            delegatable=delegatable, renewable=renewable)
        soap_message = add_saml_context(serialized_request=soap_message,
                                        saml_token=hok_token,
                                        private_key_file=private_key,
                                        request_duration=request_duration)

        hok_token = self.perform_request(soap_message=soap_message,
                                         ssl_context=ssl_context)
        return etree.tostring(_extract_element(
            etree.fromstring(hok_token), 'Assertion',
            {'saml2': "urn:oasis:names:tc:SAML:2.0:assertion"}),
                              pretty_print=False).decode(UTF_8)


class SecurityTokenRequest(object):
    """
    SecurityTokenRequest class handles the serialization of request to the STS
    for a SAML token.
    """

    #pylint: disable=R0902
    def __init__(self,
                 username=None,
                 password=None,
                 public_key=None,
                 private_key=None,
                 request_duration=60,
                 token_duration=600,
                 gss_binary_token=None,
                 hok_token=None):
        """
        Initializer for the SecurityToken Request class.

        @type          username: C{str}
        @param         username: Username for the user for which bearer token
                                 needs to be requested.
        @type          password: C{str}
        @param         password: Password for the user for which bearer token
                                 needs to be requested.
        @type        public_key: C{str}
        @param       public_key: The file containing the public key of the
                                 service account requesting the SAML token.
        @type       private_key: C{str}
        @param      private_key: The file containing the private key of the
                                 service account requesting the SAML token.
        @type  request_duration: C{long}
        @param request_duration: The duration for which the request is valid. If
                                 the STS receives this request after this
                                 duration, it is assumed to have expired. The
                                 duration is specified in seconds and default is
                                 60s.
        @type    token_duration: C{long}
        @param   token_duraiton: The duration for which the SAML token is issued
                                 for. The duration is specified in seconds and
                                 the default is 600s.
        """
        self._timestamp_id = _generate_id()
        self._signature_id = _generate_id()
        self._request_id = _generate_id()
        self._security_token_id = _generate_id()
        current = datetime.datetime.utcnow()
        self._created = format_time(current.strftime(TIME_FORMAT))
        self._expires = format_time(
            (current +
             datetime.timedelta(seconds=token_duration)).strftime(TIME_FORMAT))
        self._request_expires = format_time(
            (current + datetime.timedelta(seconds=request_duration)
             ).strftime(TIME_FORMAT))
        self._timestamp = TIMESTAMP_TEMPLATE % self.__dict__
        self._username = escape(username) if username else username
        self._password = escape(password) if password else password
        self._public_key_file = public_key
        self._private_key_file = private_key
        self._act_as_token = None
        self._renewable = str(False).lower()
        self._delegatable = str(False).lower()
        self._use_key = ""
        self._private_key = None
        self._binary_exchange = None
        self._public_key = None
        if gss_binary_token:
            self._binary_exchange = BINARY_EXCHANGE_TEMPLATE % gss_binary_token
        #The following are populated later. Set to None here to keep in-line
        #with PEP8.
        self._binary_security_token = None
        self._hok_token = hok_token
        self._key_type = None
        self._security_token = None
        self._signature_text = None
        self._signature = None
        self._signed_info = None
        self._timestamp_digest = None
        self._signature_value = None
        self._xml_text = None
        self._xml = None
        self._request_digest = None

        #These will only be populated if requesting an HoK token.
        if self._private_key_file:
            with open(self._private_key_file) as fp:
                self._private_key = fp.read()

        if self._public_key_file:
            with open(self._public_key_file) as fp:
                self._public_key = fp.read()

    def construct_bearer_token_request(self,
                                       delegatable=False,
                                       renewable=False):
        """
        Constructs the actual Bearer token SOAP request.

        @type  delegatable: C{boolean}
        @param delegatable: Whether the generated token is delegatable or not
        @type    renewable: C{boolean}
        @param   renewable: Whether the generated token is renewable or not
                            The default value is False
        @rtype:  C{str}
        @return: Bearer token SOAP request.
        """
        self._key_type = "http://docs.oasis-open.org/ws-sx/ws-trust/200512/Bearer"
        self._security_token = USERNAME_TOKEN_TEMPLATE % self.__dict__
        self._delegatable = str(delegatable).lower()
        self._renewable = str(renewable).lower()
        return _canonicalize(REQUEST_TEMPLATE % self.__dict__)

    def construct_bearer_token_request_with_binary_token(
            self, delegatable=False, renewable=False):
        """
        Constructs the actual Bearer token SOAP request using the binary exchange GSS/SSPI token.

        @type  delegatable: C{boolean}
        @param delegatable: Whether the generated token is delegatable or not
        @type    renewable: C{boolean}
        @param   renewable: Whether the generated token is renewable or not
                            The default value is False
        @rtype:  C{str}
        @return: Bearer token SOAP request.
        """
        self._key_type = "http://docs.oasis-open.org/ws-sx/ws-trust/200512/Bearer"
        self._delegatable = str(delegatable).lower()
        self._renewable = str(renewable).lower()
        return _canonicalize(GSS_REQUEST_TEMPLATE % self.__dict__)

    def construct_hok_request(self,
                              delegatable=False,
                              act_as_token=None,
                              renewable=False):
        """
        Constructs the actual HoK token SOAP request.

        @type   delegatable: C{boolean}
        @param  delegatable: Whether the generated token is delegatable or not
        @type  act_as_token: C{str}
        @param act_as_token: Bearer/Hok token which is delegatable
        @type    renewable: C{boolean}
        @param   renewable: Whether the generated token is renewable or not
                            The default value is False
        @rtype: C{str}
        @return: HoK token SOAP request in Unicode.
        """
        self._binary_security_token = base64.b64encode(
            _extract_certificate(self._public_key)).decode(UTF_8)
        self._use_key = USE_KEY_TEMPLATE % self.__dict__
        self._security_token = BINARY_SECURITY_TOKEN_TEMPLATE % self.__dict__
        self._key_type = "http://docs.oasis-open.org/ws-sx/ws-trust/200512/PublicKey"
        self._renewable = str(renewable).lower()
        self._delegatable = str(delegatable).lower()
        self._act_as_token = act_as_token
        if act_as_token is None:
            self._xml_text = _canonicalize(REQUEST_TEMPLATE % self.__dict__)
        else:
            self._xml_text = ACTAS_REQUEST_TEMPLATE % self.__dict__
        self.sign_request()
        return etree.tostring(self._xml, pretty_print=False).decode(UTF_8)

    def construct_hok_by_hok_request(self, delegatable=False, renewable=False):
        """
        @type    delegatable: C{boolean}
        @param   delegatable: Whether the generated token is delegatable or not
                            The default value is False
        @type    renewable: C{boolean}
        @param   renewable: Whether the generated token is renewable or not
                            The default value is False
        @rtype: C{str}
        @return: HoK token SOAP request in Unicode.
        """
        self._delegatable = str(delegatable).lower()
        self._renewable = str(renewable).lower()
        self._key_type = "http://docs.oasis-open.org/ws-sx/ws-trust/200512/PublicKey"
        return _canonicalize(REQUEST_TEMPLATE_TOKEN_BY_TOKEN) % self.__dict__

    def sign_request(self):
        """
        Calculates the signature to the header of the SOAP request which can be
        used by the STS to verify that the SOAP message originated from a
        trusted service.
        """
        base_xml = etree.fromstring(self._xml_text)
        request_tree = _extract_element(
            base_xml, 'Body',
            {'SOAP-ENV': "http://schemas.xmlsoap.org/soap/envelope/"})
        request = _canonicalize(etree.tostring(request_tree))
        request_tree = _extract_element(
            base_xml, 'Timestamp', {
                'ns3':
                "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd"
            })
        timestamp = _canonicalize(etree.tostring(request_tree))
        self._request_digest = _make_hash(request.encode(UTF_8)).decode(UTF_8)  # pylint: disable=W0612
        self._timestamp_digest = _make_hash(timestamp.encode(UTF_8)).decode(
            UTF_8)  # pylint: disable=W0612
        self._algorithm = SHA256
        self._signed_info = _canonicalize(SIGNED_INFO_TEMPLATE % self.__dict__)
        self._signature_value = _sign(self._private_key,
                                      self._signed_info).decode(UTF_8)
        self._signature_text = _canonicalize(SIGNATURE_TEMPLATE %
                                             self.__dict__)
        self.embed_signature()

    def embed_signature(self):
        """
        Embeds the signature in to the header of the SOAP request.
        """
        self._xml = etree.fromstring(self._xml_text)
        security = _extract_element(
            self._xml, 'Security', {
                'ns6':
                "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd"
            })
        self._signature = etree.fromstring(self._signature_text)
        security.append(self._signature)
        self._xml_text = etree.tostring(self._xml).decode(UTF_8)


def add_saml_context(serialized_request,
                     saml_token,
                     private_key_file,
                     request_duration=60):
    """
    A helper method provided to sign the outgoing LoginByToken requests with the
    HoK token.

    @type       serialized_request: C{str}
    @param      serialized_request: SOAP request which needs to be signed.
    @type               saml_token: C{str}
    @param              saml_token: SAML assertion that will be added to the SOAP
                                    request.
    @type         private_key_file: C{str}
    @param        private_key_file: Private key of the service user that will be
                                    used to sign the request, in PEM format.
    @type         request_duration: C{long}
    @param request_duration: The duration for which the request is valid. If
                                 the STS receives this request after this
                                 duration, it is assumed to have expired. The
                                 duration is in seconds and the default is 60s.
    @rtype: C{str}
    @return: signed SOAP request in Unicode.
    """
    with open(private_key_file) as fp:
        private_key = fp.read()
    xml = etree.fromstring(serialized_request)
    value_map = {}
    value_map['_request_id'] = _generate_id()
    request_body = _extract_element(
        xml, 'Body', {'soapenv': "http://schemas.xmlsoap.org/soap/envelope/"})
    request_body.nsmap[
        "wsu"] = "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd"
    request_body.set(
        "{http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd}Id",
        value_map['_request_id'])
    value_map['_request_digest'] = _make_hash_sha512(
        _canonicalize(
            etree.tostring(request_body)).encode(UTF_8)).decode(UTF_8)
    security = _extract_element(
        xml, 'Security', {
            'ns6':
            "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd"
        })
    current = datetime.datetime.utcnow()
    value_map['_created'] = format_time(current.strftime(TIME_FORMAT))
    value_map['_request_expires'] = format_time(
        (current +
         datetime.timedelta(seconds=request_duration)).strftime(TIME_FORMAT))
    value_map['_timestamp_id'] = _generate_id()
    timestamp = _canonicalize(TIMESTAMP_TEMPLATE % value_map)
    value_map['_timestamp_digest'] = _make_hash_sha512(
        timestamp.encode(UTF_8)).decode(UTF_8)

    security.append(etree.fromstring(timestamp))
    value_map['_algorithm'] = SHA512
    value_map['_signed_info'] = _canonicalize(SIGNED_INFO_TEMPLATE % value_map)
    value_map['_signature_value'] = _sign(private_key,
                                          value_map['_signed_info'],
                                          SHA512).decode(UTF_8)
    value_map['samlId'] = etree.fromstring(saml_token).get("ID")
    signature = etree.fromstring(
        _canonicalize(REQUEST_SIGNATURE_TEMPLATE % value_map))
    security.append(signature)
    return etree.tostring(xml, pretty_print=False).decode(UTF_8)


def _generate_id():
    """
    An internal helper method to generate UUIDs.

    @rtype: C{str}
    @return: UUID
    """
    return "_%s" % uuid4()


def _load_private_key(der_key):
    """
    An internal helper to load private key.

    @type  der_key: C{str}
    @param der_key: The private key, in DER format.

    @rtype: crypto.privatekey
    @return: Loaded private key.
    """

    # OpenSSL 0.9.8 does not handle correctly PKCS8 keys passed in DER format
    # (only PKCS1 keys are understood in DER format).

    # Unencrypted PKCS8, or PKCS1 for OpenSSL 1.0.1, PKCS1 for OpenSSL 0.9.8
    try:
        return crypto.load_privatekey(crypto.FILETYPE_ASN1, der_key, b'')
    except (crypto.Error, ValueError):
        pass
    # Unencrypted PKCS8 for OpenSSL 0.9.8, and PKCS1, just in case...
    for key_type in ('PRIVATE KEY', 'RSA PRIVATE KEY'):
        try:
            return crypto.load_privatekey(
                crypto.FILETYPE_PEM, '-----BEGIN ' + key_type + '-----\n' +
                base64.encodebytes(der_key).decode(UTF_8) + '-----END ' +
                key_type + '-----\n', b'')
        except (crypto.Error, ValueError):
            pass
    # We could try 'ENCRYPTED PRIVATE KEY' here - but we do not know passphrase.
    raise


def _sign(private_key, data, digest=SHA256):
    """
    An internal helper method to sign the 'data' with the 'private_key'.

    @type  private_key: C{str}
    @param private_key: The private key used to sign the 'data', in one of
                        supported formats.
    @type         data: C{str}
    @param        data: The data that needs to be signed.
    @type       digest: C{str}
    @param      digest: Digest is a str naming a supported message digest type,
                        for example 'sha256'.

    @rtype: C{str}
    @return: Signed string.
    """
    # Convert private key in arbitrary format into DER (DER is binary format
    # so we get rid of \n / \r\n differences, and line breaks in PEM).
    pkey = _load_private_key(_extract_certificate(private_key))
    return base64.b64encode(crypto.sign(pkey, data.encode(UTF_8), digest))


def _canonicalize(xml_string):
    """
    Given an xml string, canonicalize the string per
    U{http://www.w3.org/2001/10/xml-exc-c14n#}

    @type  xml_string: C{str}
    @param xml_string: The XML string that needs to be canonicalized.

    @rtype: C{str}
    @return: Canonicalized string in Unicode.
    """
    # TODO: keep the parser between _canonicalize() invocations.
    parser = etree.XMLParser(remove_blank_text=True, resolve_entities=False)
    tree = etree.fromstring(xml_string, parser=parser).getroottree()
    string = BytesIO()
    tree.write_c14n(string, exclusive=True, with_comments=False)
    return string.getvalue().decode(UTF_8)


def _extract_element(xml, element_name, namespace):
    """
    An internal method provided to extract an element from the given XML.

    @type           xml: C{str}
    @param          xml: The XML string from which the element will be extracted.
    @type  element_name: C{str}
    @param element_name: The element that needs to be extracted from the XML.
    @type     namespace: dict
    @param    namespace: A dict containing the namespace of the element to be
                         extracted.

    @rtype: etree element.
    @return: The extracted element.
    """
    assert (len(namespace) == 1)
    result = xml.xpath("//{0}:{1}".format(list(namespace.keys())[0], element_name),
                       namespaces=namespace)
    if result:
        return result[0]
    else:
        raise KeyError("%s does not seem to be present in the XML." %
                       element_name)


def _make_hash(data):
    """
    An internal method to calculate the sha256 hash of the data.

    @type  data: C{str}
    @param data: The data for which the hash needs to be calculated.

    @rtype: C{str}
    @return: Base64 encoded sha256 hash.
    """
    return base64.b64encode(hashlib.sha256(data).digest())  # pylint: disable=E1101


def _make_hash_sha512(data):
    """
    An internal method to calculate the sha512 hash of the data.

    @type  data:      C{str}
    @param data:      The data for which the hash needs to be calculated.

    @rtype: C{str}
    @return: Base64 encoded sha512 hash.
    """
    return base64.b64encode(hashlib.sha512(data).digest())  # pylint: disable=E1101


TIME_FORMAT = "%Y-%m-%dT%H:%M:%S.%f"

# The SAML token requests usually contain an xmldsig which guarantees that the
# message hasn't been tampered with during the transport. The following
# SIGNED_INFO_TEMPLATE is used to construct the signedinfo part of the signature.
SIGNED_INFO_TEMPLATE = """\
<ds:SignedInfo xmlns:ds="http://www.w3.org/2000/09/xmldsig#">
<ds:CanonicalizationMethod Algorithm="http://www.w3.org/2001/10/xml-exc-c14n#"/>
<ds:SignatureMethod Algorithm="http://www.w3.org/2001/04/xmldsig-more#rsa-%(_algorithm)s"/>
<ds:Reference URI="#%(_request_id)s">
<ds:Transforms>
<ds:Transform Algorithm="http://www.w3.org/2001/10/xml-exc-c14n#"/>
</ds:Transforms>
<ds:DigestMethod Algorithm="http://www.w3.org/2001/04/xmlenc#%(_algorithm)s"/>
<ds:DigestValue>%(_request_digest)s</ds:DigestValue>
</ds:Reference>
<ds:Reference URI="#%(_timestamp_id)s">
<ds:Transforms>
<ds:Transform Algorithm="http://www.w3.org/2001/10/xml-exc-c14n#"/>
</ds:Transforms>
<ds:DigestMethod Algorithm="http://www.w3.org/2001/04/xmlenc#%(_algorithm)s"/>
<ds:DigestValue>%(_timestamp_digest)s</ds:DigestValue>
</ds:Reference>
</ds:SignedInfo>
"""

# The following template is used as the container for signed info in WS-Trust
# SOAP requests signed with the SAML token. It contains the digest of the
# signed info, signed with the private key of the Solution user and contains a
# reference to the actual SAML token which contains the solution user's public
# key.
REQUEST_SIGNATURE_TEMPLATE = """\
<ds:Signature xmlns:ds="http://www.w3.org/2000/09/xmldsig#">
%(_signed_info)s
<ds:SignatureValue>%(_signature_value)s</ds:SignatureValue>
<ds:KeyInfo>
<ns2:SecurityTokenReference xmlns:ns2="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd"
                            xmlns:wsse11="http://docs.oasis-open.org/wss/oasis-wss-wssecurity-secext-1.1.xsd"
                            wsse11:TokenType="http://docs.oasis-open.org/wss/oasis-wss-saml-token-profile-1.1#SAMLV2.0">
<ns2:KeyIdentifier ValueType="http://docs.oasis-open.org/wss/oasis-wss-saml-token-profile-1.1#SAMLID">%(samlId)s</ns2:KeyIdentifier>
</ns2:SecurityTokenReference>
</ds:KeyInfo>
</ds:Signature>"""

# The following template is used as a signed info container for the actual SAML
# token requests requesting a SAML token. It contains the digest of the signed
# info signed with the Service User's private key.
SIGNATURE_TEMPLATE = """\
<ds:Signature xmlns:ds="http://www.w3.org/2000/09/xmldsig#" Id="%(_signature_id)s">
%(_signed_info)s
<ds:SignatureValue>%(_signature_value)s</ds:SignatureValue>
<ds:KeyInfo>
<ns2:SecurityTokenReference xmlns:ns2="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd">
<ns2:Reference URI="#%(_security_token_id)s" ValueType="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509v3"/>
</ns2:SecurityTokenReference>
</ds:KeyInfo>
</ds:Signature>"""

# The following template is used to construct the token requests to the STS.
REQUEST_TEMPLATE = """\
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<SOAP-ENV:Header>
<ns6:Security xmlns="http://docs.oasis-open.org/ws-sx/ws-trust/200512"
              xmlns:ns2="http://www.w3.org/2005/08/addressing"
              xmlns:ns3="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd"
              xmlns:ns6="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd">
%(_timestamp)s
%(_security_token)s
</ns6:Security>
</SOAP-ENV:Header>
<SOAP-ENV:Body xmlns:wsu="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd" wsu:Id="%(_request_id)s">
<RequestSecurityToken xmlns="http://docs.oasis-open.org/ws-sx/ws-trust/200512"
                      xmlns:ns2="http://www.w3.org/2005/08/addressing"
                      xmlns:ns3="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd"
                      xmlns:ns6="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd">
<TokenType>urn:oasis:names:tc:SAML:2.0:assertion</TokenType>
<RequestType>http://docs.oasis-open.org/ws-sx/ws-trust/200512/Issue</RequestType>
<Lifetime>
<ns3:Created>%(_created)s</ns3:Created>
<ns3:Expires>%(_expires)s</ns3:Expires>
</Lifetime>
<Renewing Allow="%(_renewable)s" OK="%(_renewable)s"/>
<Delegatable>%(_delegatable)s</Delegatable>
<KeyType>%(_key_type)s</KeyType>
<SignatureAlgorithm>http://www.w3.org/2001/04/xmldsig-more#rsa-sha256</SignatureAlgorithm>%(_use_key)s</RequestSecurityToken>
</SOAP-ENV:Body>
</SOAP-ENV:Envelope>"""

# The following template is used to construct the token-by-token requests to the STS.
REQUEST_TEMPLATE_TOKEN_BY_TOKEN = """\
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<SOAP-ENV:Header>
<ns5:Security xmlns="http://docs.oasis-open.org/ws-sx/ws-trust/200512"
              xmlns:ns3="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd"
              xmlns:ns5="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd">
%(_hok_token)s
</ns5:Security>
</SOAP-ENV:Header>
<SOAP-ENV:Body xmlns:wsu="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd" wsu:Id="%(_request_id)s">
<RequestSecurityToken xmlns="http://docs.oasis-open.org/ws-sx/ws-trust/200512"
                      xmlns:ns3="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd">
<TokenType>urn:oasis:names:tc:SAML:2.0:assertion</TokenType>
<RequestType>http://docs.oasis-open.org/ws-sx/ws-trust/200512/Issue</RequestType>
<Lifetime>
<ns3:Created>%(_created)s</ns3:Created>
<ns3:Expires>%(_expires)s</ns3:Expires>
</Lifetime>
<Renewing Allow="%(_renewable)s" OK="%(_renewable)s"/>
<Delegatable>%(_delegatable)s</Delegatable>
<KeyType>%(_key_type)s</KeyType>
<SignatureAlgorithm>http://www.w3.org/2001/04/xmldsig-more#rsa-sha256</SignatureAlgorithm>
</RequestSecurityToken>
</SOAP-ENV:Body>
</SOAP-ENV:Envelope>"""

GSS_REQUEST_TEMPLATE = """\
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<SOAP-ENV:Header>
<ns6:Security xmlns="http://docs.oasis-open.org/ws-sx/ws-trust/200512"
              xmlns:ns2="http://www.w3.org/2005/08/addressing"
              xmlns:ns3="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd"
              xmlns:ns6="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd">
%(_timestamp)s
</ns6:Security>
</SOAP-ENV:Header>
<SOAP-ENV:Body xmlns:wsu="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd" wsu:Id="%(_request_id)s">
<RequestSecurityToken xmlns="http://docs.oasis-open.org/ws-sx/ws-trust/200512"
                      xmlns:ns2="http://www.w3.org/2005/08/addressing"
                      xmlns:ns3="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd"
                      xmlns:ns6="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd">
<TokenType>urn:oasis:names:tc:SAML:2.0:assertion</TokenType>
<RequestType>http://docs.oasis-open.org/ws-sx/ws-trust/200512/Issue</RequestType>
<Lifetime>
<ns3:Created>%(_created)s</ns3:Created>
<ns3:Expires>%(_expires)s</ns3:Expires>
</Lifetime>
<Renewing Allow="%(_renewable)s" OK="%(_renewable)s"/>
<Delegatable>%(_delegatable)s</Delegatable>
<KeyType>%(_key_type)s</KeyType>
<SignatureAlgorithm>http://www.w3.org/2001/04/xmldsig-more#rsa-sha256</SignatureAlgorithm>
%(_binary_exchange)s
%(_use_key)s</RequestSecurityToken>
</SOAP-ENV:Body>
</SOAP-ENV:Envelope>"""

# Template container for the service user's public key when requesting an HoK
# token.
BINARY_SECURITY_TOKEN_TEMPLATE = """\
<ns2:BinarySecurityToken xmlns:ns1="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd"
                         xmlns:ns2="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd"
                         EncodingType="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-soap-message-security-1.0#Base64Binary"
                         ValueType="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509v3"
                         ns1:Id="%(_security_token_id)s">%(_binary_security_token)s</ns2:BinarySecurityToken>
"""

# Template container for user's credentials when requesting a bearer token.
USERNAME_TOKEN_TEMPLATE = """\
<ns2:UsernameToken xmlns:ns2="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd">
<ns2:Username>%(_username)s</ns2:Username>
<ns2:Password>%(_password)s</ns2:Password>
</ns2:UsernameToken>"""

# Template containing the anchor to the signature.
USE_KEY_TEMPLATE = """\
<UseKey Sig="%(_signature_id)s"/>"""

# The follwoing template is used to create a timestamp for the various messages.
# The timestamp is used to indicate the duration of the request itself.
TIMESTAMP_TEMPLATE = """\
<ns3:Timestamp xmlns:ns3="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd" ns3:Id="%(_timestamp_id)s">
<ns3:Created>%(_created)s</ns3:Created><ns3:Expires>%(_request_expires)s</ns3:Expires></ns3:Timestamp>"""

BINARY_EXCHANGE_TEMPLATE = """\
<BinaryExchange EncodingType="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-soap-message-security-1.0#Base64Binary" ValueType="http://schemas.xmlsoap.org/ws/2005/02/trust/spnego">%s</BinaryExchange>"""

ACTAS_REQUEST_TEMPLATE = """<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"><SOAP-ENV:Header><ns6:Security xmlns:ns6="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd"><ns3:Timestamp xmlns:ns3="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd" ns3:Id="%(_timestamp_id)s"><ns3:Created>%(_created)s</ns3:Created><ns3:Expires>%(_request_expires)s</ns3:Expires></ns3:Timestamp><ns2:BinarySecurityToken xmlns:ns1="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd" xmlns:ns2="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd" EncodingType="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-soap-message-security-1.0#Base64Binary" ValueType="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509v3" ns1:Id="%(_security_token_id)s">%(_binary_security_token)s</ns2:BinarySecurityToken></ns6:Security></SOAP-ENV:Header><SOAP-ENV:Body xmlns:wsu="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd" wsu:Id="%(_request_id)s"><RequestSecurityToken xmlns="http://docs.oasis-open.org/ws-sx/ws-trust/200512"><TokenType>urn:oasis:names:tc:SAML:2.0:assertion</TokenType><RequestType>http://docs.oasis-open.org/ws-sx/ws-trust/200512/Issue</RequestType><Lifetime><ns3:Created xmlns:ns3="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd">%(_created)s</ns3:Created><ns3:Expires xmlns:ns3="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd">%(_expires)s</ns3:Expires></Lifetime><Renewing Allow="%(_renewable)s" OK="%(_renewable)s"></Renewing><Delegatable>%(_delegatable)s</Delegatable><ns4:ActAs xmlns:ns4="http://docs.oasis-open.org/ws-sx/ws-trust/200802">%(_act_as_token)s</ns4:ActAs><KeyType>http://docs.oasis-open.org/ws-sx/ws-trust/200512/PublicKey</KeyType><SignatureAlgorithm>http://www.w3.org/2001/04/xmldsig-more#rsa-sha256</SignatureAlgorithm><ns3:UseKey xmlns:ns3="http://docs.oasis-open.org/ws-sx/ws-trust/200512" Sig="%(_signature_id)s"></ns3:UseKey></RequestSecurityToken></SOAP-ENV:Body></SOAP-ENV:Envelope>"""



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.alarm import AlarmAction
from pyVmomi.vim.alarm import AlarmExpression
from pyVmomi.vim.alarm import AlarmSetting

class AlarmSpec(DynamicData):
   name: str
   systemName: Optional[str] = None
   description: str
   enabled: bool
   expression: AlarmExpression
   action: Optional[AlarmAction] = None
   actionFrequency: Optional[int] = None
   setting: Optional[AlarmSetting] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vim import HostSystem

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.vsan.host import DiskMapInfo
from pyVmomi.vim.vsan.host import DiskMapping
from pyVmomi.vim.vsan.host import IpConfig

class ConfigInfo(DynamicData):
   class StorageInfo(DynamicData):
      autoClaimStorage: Optional[bool] = None
      diskMapping: list[DiskMapping] = []
      diskMapInfo: list[DiskMapInfo] = []
      checksumEnabled: Optional[bool] = None

   class ClusterInfo(DynamicData):
      uuid: Optional[str] = None
      nodeUuid: Optional[str] = None

   class NetworkInfo(DynamicData):
      class PortConfig(DynamicData):
         ipConfig: Optional[IpConfig] = None
         device: str

      port: list[PortConfig] = []

   class FaultDomainInfo(DynamicData):
      name: str

   enabled: Optional[bool] = None
   hostSystem: Optional[HostSystem] = None
   clusterInfo: Optional[ClusterInfo] = None
   storageInfo: Optional[StorageInfo] = None
   networkInfo: Optional[NetworkInfo] = None
   faultDomainInfo: Optional[FaultDomainInfo] = None
   vsanEsaEnabled: Optional[bool] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vim import Datastore

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.cns import BackingObjectDetails
from pyVmomi.vim.cns import BaseCreateSpec
from pyVmomi.vim.cns import VolumeMetadata
from pyVmomi.vim.cns import VolumeSource

from pyVmomi.vim.vm import ProfileSpec

class VolumeCreateSpec(DynamicData):
   name: str
   volumeType: str
   datastores: list[Datastore] = []
   metadata: Optional[VolumeMetadata] = None
   backingObjectDetails: BackingObjectDetails
   profile: list[ProfileSpec] = []
   createSpec: Optional[BaseCreateSpec] = None
   volumeSource: Optional[VolumeSource] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.sms import EntityReference

from pyVmomi.vmodl.fault import InvalidArgument

class QueryNotSupported(InvalidArgument):
   entityType: Optional[EntityReference.EntityType] = None
   relatedEntityType: EntityReference.EntityType



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import Optional

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vim import IntExpression
from pyVmomi.vim import IpAddress
from pyVmomi.vim import MacAddress
from pyVmomi.vim import NegatableExpression
from pyVmomi.vim import SingleIp
from pyVmomi.vim import StringExpression

from pyVmomi.vmodl import DynamicData

class TrafficRule(DynamicData):
   class Qualifier(DynamicData):
      key: Optional[str] = None

   class Action(DynamicData):
      pass

   class RuleDirectionType(Enum):
      incomingPackets: ClassVar['RuleDirectionType'] = 'incomingPackets'
      outgoingPackets: ClassVar['RuleDirectionType'] = 'outgoingPackets'
      both: ClassVar['RuleDirectionType'] = 'both'

   class IpQualifier(Qualifier):
      sourceAddress: Optional[IpAddress] = None
      destinationAddress: Optional[IpAddress] = None
      protocol: Optional[IntExpression] = None
      sourceIpPort: Optional[IpPort] = None
      destinationIpPort: Optional[IpPort] = None
      tcpFlags: Optional[IntExpression] = None

   class IpPort(NegatableExpression):
      pass

   class SingleIpPort(IpPort):
      portNumber: int

   class IpPortRange(IpPort):
      startPortNumber: int
      endPortNumber: int

   class MacQualifier(Qualifier):
      sourceAddress: Optional[MacAddress] = None
      destinationAddress: Optional[MacAddress] = None
      protocol: Optional[IntExpression] = None
      vlanId: Optional[IntExpression] = None

   class SystemTrafficQualifier(Qualifier):
      typeOfSystemTraffic: Optional[StringExpression] = None

   class DropAction(Action):
      pass

   class AcceptAction(Action):
      pass

   class UpdateTagAction(Action):
      qosTag: Optional[int] = None
      dscpTag: Optional[int] = None

   class RateLimitAction(Action):
      packetsPerSecond: int

   class LogAction(Action):
      pass

   class GreAction(Action):
      encapsulationIp: SingleIp

   class MacRewriteAction(Action):
      rewriteMac: str

   class PuntAction(Action):
      pass

   class CopyAction(Action):
      pass

   key: Optional[str] = None
   description: Optional[str] = None
   sequence: Optional[int] = None
   qualifier: list[Qualifier] = []
   action: Optional[Action] = None
   direction: Optional[str] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vim import Description

from pyVmomi.vmodl import DynamicData

class VirtualDeviceGroups(DynamicData):
   class DeviceGroup(DynamicData):
      groupInstanceKey: int
      deviceInfo: Optional[Description] = None

   class VendorDeviceGroup(DeviceGroup):
      deviceGroupName: str

   deviceGroup: list[DeviceGroup] = []



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import NoReturn
from typing import Optional

from pyVmomi.vim import ExtensibleManagedObject

from pyVmomi.vim.host import DnsConfig
from pyVmomi.vim.host import IpRouteConfig
from pyVmomi.vim.host import IpRouteTableConfig
from pyVmomi.vim.host import NetCapabilities
from pyVmomi.vim.host import NetOffloadCapabilities
from pyVmomi.vim.host import NetworkConfig
from pyVmomi.vim.host import NetworkInfo
from pyVmomi.vim.host import PhysicalNic
from pyVmomi.vim.host import PortGroup
from pyVmomi.vim.host import VirtualNic
from pyVmomi.vim.host import VirtualSwitch

class NetworkSystem(ExtensibleManagedObject):
   @property
   def capabilities(self) -> Optional[NetCapabilities]: ...
   @property
   def networkInfo(self) -> Optional[NetworkInfo]: ...
   @property
   def offloadCapabilities(self) -> Optional[NetOffloadCapabilities]: ...
   @property
   def networkConfig(self) -> Optional[NetworkConfig]: ...
   @property
   def dnsConfig(self) -> Optional[DnsConfig]: ...
   @property
   def ipRouteConfig(self) -> Optional[IpRouteConfig]: ...
   @property
   def consoleIpRouteConfig(self) -> Optional[IpRouteConfig]: ...

   def UpdateNetworkConfig(self, config: NetworkConfig, changeMode: str) -> NetworkConfig.Result: ...
   def UpdateDnsConfig(self, config: DnsConfig) -> NoReturn: ...
   def UpdateIpRouteConfig(self, config: IpRouteConfig) -> NoReturn: ...
   def UpdateConsoleIpRouteConfig(self, config: IpRouteConfig) -> NoReturn: ...
   def UpdateIpRouteTableConfig(self, config: IpRouteTableConfig) -> NoReturn: ...
   def AddVirtualSwitch(self, vswitchName: str, spec: Optional[VirtualSwitch.Specification]) -> NoReturn: ...
   def RemoveVirtualSwitch(self, vswitchName: str) -> NoReturn: ...
   def UpdateVirtualSwitch(self, vswitchName: str, spec: VirtualSwitch.Specification) -> NoReturn: ...
   def AddPortGroup(self, portgrp: PortGroup.Specification) -> NoReturn: ...
   def RemovePortGroup(self, pgName: str) -> NoReturn: ...
   def UpdatePortGroup(self, pgName: str, portgrp: PortGroup.Specification) -> NoReturn: ...
   def UpdatePhysicalNicLinkSpeed(self, device: str, linkSpeed: Optional[PhysicalNic.LinkSpeedDuplex]) -> NoReturn: ...
   def QueryNetworkHint(self, device: list[str]) -> list[PhysicalNic.NetworkHint]: ...
   def AddVirtualNic(self, portgroup: str, nic: VirtualNic.Specification) -> str: ...
   def RemoveVirtualNic(self, device: str) -> NoReturn: ...
   def UpdateVirtualNic(self, device: str, nic: VirtualNic.Specification) -> NoReturn: ...
   def AddServiceConsoleVirtualNic(self, portgroup: str, nic: VirtualNic.Specification) -> str: ...
   def RemoveServiceConsoleVirtualNic(self, device: str) -> NoReturn: ...
   def UpdateServiceConsoleVirtualNic(self, device: str, nic: VirtualNic.Specification) -> NoReturn: ...
   def RestartServiceConsoleVirtualNic(self, device: str) -> NoReturn: ...
   def Refresh(self) -> NoReturn: ...
   def StartDpuFailover(self, dvsName: str, targetDpuAlias: Optional[str]) -> NoReturn: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.host import FibreChannelHba

class FibreChannelOverEthernetHba(FibreChannelHba):
   class LinkInfo(DynamicData):
      vnportMac: str
      fcfMac: str
      vlanId: int

   underlyingNic: str
   linkInfo: LinkInfo
   isSoftwareFcoe: bool
   markedForRemoval: Optional[bool] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from pyVmomi.VmomiSupport import long
from pyVmomi.VmomiSupport import short

from pyVmomi.vmodl import DynamicData

class CpuInfo(DynamicData):
   numCpuPackages: short
   numCpuCores: short
   numCpuThreads: short
   hz: long



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vmodl import DynamicData

class NetCapabilities(DynamicData):
   canSetPhysicalNicLinkSpeed: bool
   supportsNicTeaming: bool
   nicTeamingPolicy: list[str] = []
   supportsVlan: bool
   usesServiceConsoleNic: bool
   supportsNetworkHints: bool
   maxPortGroupsPerVswitch: Optional[int] = None
   vswitchConfigSupported: bool
   vnicConfigSupported: bool
   ipRouteConfigSupported: bool
   dnsConfigSupported: bool
   dhcpOnVnicSupported: bool
   ipV6Supported: bool
   backupNfcNiocSupported: Optional[bool] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vim import ImportSpec
from pyVmomi.vim import ResourceConfigSpec

from pyVmomi.vim.vApp import VAppConfigSpec

class VAppImportSpec(ImportSpec):
   name: str
   vAppConfigSpec: VAppConfigSpec
   resourcePoolSpec: ResourceConfigSpec
   child: list[ImportSpec] = []



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import Optional

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.host import NetworkPolicy
from pyVmomi.vim.host import VirtualSwitch

class PortGroup(DynamicData):
   class PortConnecteeType(Enum):
      virtualMachine: ClassVar['PortConnecteeType'] = 'virtualMachine'
      systemManagement: ClassVar['PortConnecteeType'] = 'systemManagement'
      host: ClassVar['PortConnecteeType'] = 'host'
      unknown: ClassVar['PortConnecteeType'] = 'unknown'

   class Specification(DynamicData):
      name: str
      vlanId: int
      vswitchName: str
      policy: NetworkPolicy

   class Config(DynamicData):
      changeOperation: Optional[str] = None
      spec: Optional[Specification] = None

   class Port(DynamicData):
      key: Optional[str] = None
      mac: list[str] = []
      type: str

   key: Optional[str] = None
   port: list[Port] = []
   vswitch: Optional[VirtualSwitch] = None
   computedPolicy: NetworkPolicy
   spec: Specification



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import Optional

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vim import HostSystem

from pyVmomi.vmodl import DynamicData

class ProtocolEndpoint(DynamicData):
   class PEType(Enum):
      block: ClassVar['PEType'] = 'block'
      nas: ClassVar['PEType'] = 'nas'

   class ProtocolEndpointType(Enum):
      scsi: ClassVar['ProtocolEndpointType'] = 'scsi'
      nfs: ClassVar['ProtocolEndpointType'] = 'nfs'
      nfs4x: ClassVar['ProtocolEndpointType'] = 'nfs4x'

   peType: str
   type: Optional[str] = None
   uuid: str
   hostKey: list[HostSystem] = []
   storageArray: Optional[str] = None
   nfsServer: Optional[str] = None
   nfsDir: Optional[str] = None
   nfsServerScope: Optional[str] = None
   nfsServerMajor: Optional[str] = None
   nfsServerAuthType: Optional[str] = None
   nfsServerUser: Optional[str] = None
   deviceId: Optional[str] = None
   usedByStretchedContainer: Optional[bool] = None



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Copyright (c) 2016-2024 Broadcom. All Rights Reserved.
The term "Broadcom" refers to Broadcom Inc. and/or its subsidiaries.

This module defines basic helper functions for vSAN-based development.
"""

__author__ = 'Broadcom, Inc'

import sys
import ssl
if (sys.version_info[0] == 3):
   from urllib.request import urlopen
else:
   from urllib2 import urlopen
from xml.dom import minidom

from pyVmomi import vim, vmodl, SoapStubAdapter, VmomiSupport
# Import the vSAN API python bindings
import pyVmomi
import vsanmgmtObjects

VSAN_API_VC_SERVICE_ENDPOINT = '/vsanHealth'
VSAN_API_ESXI_SERVICE_ENDPOINT = '/vsan'

VSAN_VMODL_VERSION = "vsan.version.version3"

# Construct a stub for vSAN API access using vCenter or ESXi sessions from
# existing stubs. Corresponding vCenter or ESXi service endpoint is required.
# vCenter service endpoint is used by default.
def valid_ipv6(addr):
   import socket
   try:
      socket.inet_pton(socket.AF_INET6, addr)
   except socket.error:
      return False
   return True

def _GetVsanStub(
      stub, endpoint=VSAN_API_VC_SERVICE_ENDPOINT,
      context=None, version='vim.version.version11'
   ):
   index = stub.host.rfind(':')
   if valid_ipv6(stub.host[:index][1:-1]):
      hostname = stub.host[:index][1:-1]
   else:
      hostname = stub.host[:index]
   port = int(stub.host.split(":")[-1])
   vsanStub = SoapStubAdapter(
      host=hostname,
      port=port,
      path=endpoint,
      version=version,
      sslContext=context
   )
   vsanStub.cookie = stub.cookie
   return vsanStub

# Construct a stub for access vCenter side vSAN APIs.
def GetVsanVcStub(stub, context=None, version=VSAN_VMODL_VERSION):
   return _GetVsanStub(stub, endpoint=VSAN_API_VC_SERVICE_ENDPOINT,
                       context=context, version=version)

# Construct a stub for access ESXi side vSAN APIs.
def GetVsanEsxStub(stub, context=None, version=VSAN_VMODL_VERSION):
   return _GetVsanStub(stub, endpoint=VSAN_API_ESXI_SERVICE_ENDPOINT,
                       context=context, version=version)

# Construct a stub for access ESXi side vSAN APIs.
def GetVsanVcMos(vcStub, context=None, version=VSAN_VMODL_VERSION):
   vsanStub = GetVsanVcStub(vcStub, context, version=version)
   vcMos = {
      'vsan-disk-management-system' : vim.cluster.VsanVcDiskManagementSystem(
                                         'vsan-disk-management-system',
                                         vsanStub
                                      ),
      'vsan-stretched-cluster-system' : vim.cluster.VsanVcStretchedClusterSystem(
                                           'vsan-stretched-cluster-system',
                                           vsanStub
                                        ),
      'vsan-cluster-config-system' : vim.cluster.VsanVcClusterConfigSystem(
                                        'vsan-cluster-config-system',
                                        vsanStub
                                     ),
      'vsan-performance-manager' : vim.cluster.VsanPerformanceManager(
                                      'vsan-performance-manager',
                                      vsanStub
                                   ),
      'vsan-cluster-health-system' : vim.cluster.VsanVcClusterHealthSystem(
                                        'vsan-cluster-health-system',
                                        vsanStub
                                     ),
      'vsan-upgrade-systemex' : vim.VsanUpgradeSystemEx(
                                   'vsan-upgrade-systemex',
                                    vsanStub
                                ),
      'vsan-cluster-space-report-system' : vim.cluster.VsanSpaceReportSystem(
                                              'vsan-cluster-space-report-system',
                                              vsanStub
                                           ),

      'vsan-cluster-object-system' : vim.cluster.VsanObjectSystem(
                                        'vsan-cluster-object-system',
                                        vsanStub
                                        ),
      'vsan-cluster-iscsi-target-system' : vim.cluster.VsanIscsiTargetSystem(
                                              'vsan-cluster-iscsi-target-system',
                                              vsanStub
                                           ),
      'vsan-vcsa-deployer-system' : vim.host.VsanVcsaDeployerSystem(
                                       'vsan-vcsa-deployer-system',
                                       vsanStub
                                       ),
      'vsan-vds-system' : vim.vsan.VsanVdsSystem('vsan-vds-system', vsanStub),
      'vsan-vc-capability-system' : vim.cluster.VsanCapabilitySystem(
                                       'vsan-vc-capability-system', vsanStub),
      'vsan-mass-collector' : vim.VsanMassCollector('vsan-mass-collector',
                                 vsanStub),
      'vsan-phonehome-system' : vim.VsanPhoneHomeSystem('vsan-phonehome-system',
                                   vsanStub),
      'vsan-vum-system' : vim.cluster.VsanVumSystem('vsan-vum-system', vsanStub),
      'vsan-cluster-resource-check-system': vim.vsan.VsanResourceCheckSystem(
                                           'vsan-cluster-resource-check-system',
                                           vsanStub),
      'cns-volume-manager': vim.cns.VolumeManager('cns-volume-manager',
                                                  vsanStub
                                                  ),
      'vsan-cluster-file-service-system': vim.vsan.VsanFileServiceSystem(
                                             'vsan-cluster-file-service-system',
                                             vsanStub
                                             ),
      'vsan-remote-datastore-system': vim.cluster.VsanRemoteDatastoreSystem(
                                         'vsan-remote-datastore-system',
                                         vsanStub),
      'vsan-cluster-ioinsight-manager': vim.cluster.VsanIoInsightManager(
                                         'vsan-cluster-ioinsight-manager',
                                         vsanStub),
      'vsan-cluster-diagnostics-system': vim.cluster.VsanDiagnosticsSystem(
                                         'vsan-cluster-diagnostics-system',
                                         vsanStub),
      'vsan-cluster-power-system': vim.cluster.VsanClusterPowerSystem(
                                         'vsan-cluster-power-system',
                                         vsanStub),
   }

   return vcMos

# Construct a stub for access ESXi side vSAN APIs.
def GetVsanEsxMos(esxStub, context=None, version=VSAN_VMODL_VERSION):
   vsanStub = GetVsanEsxStub(esxStub, context, version=version)
   esxMos = {
      'vsan-performance-manager' : vim.cluster.VsanPerformanceManager(
                                      'vsan-performance-manager',
                                      vsanStub
                                   ),
      'vsan-cluster-health-system' : vim.cluster.VsanVcClusterHealthSystem(
                                        'vsan-cluster-health-system',
                                        vsanStub
                                     ),
      'ha-vsan-health-system' : vim.host.VsanHealthSystem(
                                        'ha-vsan-health-system',
                                        vsanStub
                                     ),
      'vsan-object-system' : vim.cluster.VsanObjectSystem(
                                        'vsan-object-system',
                                        vsanStub
                                     ),
      'vsan-vcsa-deployer-system' : vim.host.VsanVcsaDeployerSystem(
                                       'vsan-vcsa-deployer-system',
                                       vsanStub
                                       ),
      'vsan-capability-system' : vim.cluster.VsanCapabilitySystem(
                                       'vsan-capability-system', vsanStub),
      'vsanSystem' : vim.host.VsanSystem('vsanSystem', vsanStub),
      'vsanSystemEx' : vim.host.VsanSystemEx('vsanSystemEx', vsanStub),
      'vsan-update-manager' : vim.host.VsanUpdateManager('vsan-update-manager',
                                                         vsanStub),
      'vsan-cluster-iscsi-target-system' : vim.cluster.VsanIscsiTargetSystem(
                                              'vsan-cluster-iscsi-target-system',
                                              vsanStub
                                           ),
      'vsan-host-ioinsight-manager': vim.cluster.VsanIoInsightManager(
                                         'vsan-host-ioinsight-manager',
                                         vsanStub),
   }
   return esxMos

# Convert a vSAN Task to a Task MO binding to vCenter service.
def ConvertVsanTaskToVcTask(vsanTask, vcStub):
  vcTask = vim.Task(vsanTask._moId, vcStub)
  return vcTask

# Wait for the vCenter task and returns after tasks are completed.
def WaitForTasks(tasks, si):
   pc = si.content.propertyCollector
   taskList = [str(task) for task in tasks]

   # Create filter
   objSpecs = [vmodl.query.PropertyCollector.ObjectSpec(obj=task)
         for task in tasks]
   propSpec = vmodl.query.PropertyCollector.PropertySpec(
         type=vim.Task, pathSet=[], all=True)
   filterSpec = vmodl.query.PropertyCollector.FilterSpec()
   filterSpec.objectSet = objSpecs
   filterSpec.propSet = [propSpec]
   filter = pc.CreateFilter(filterSpec, True)

   try:
      version, state = None, None

      # Loop looking for updates till the state moves to a completed state.
      while len(taskList):
         update = pc.WaitForUpdates(version)
         for filterSet in update.filterSet:
            for objSet in filterSet.objectSet:
               task = objSet.obj
               for change in objSet.changeSet:
                  if change.name == 'info':
                     state = change.val.state
                  elif change.name == 'info.state':
                     state = change.val
                  else:
                     continue

                  if not str(task) in taskList:
                     continue

                  if state == vim.TaskInfo.State.success:
                     # Remove task from taskList
                     taskList.remove(str(task))
                  elif state == vim.TaskInfo.State.error:
                     raise task.info.error
         # Move to next version
         version = update.version
   finally:
      if filter:
         filter.Destroy()

def getVsanVersionFromNamespace(versionId, localVersion):
   versionKey = "%s/%s" % ("vsan", str(versionId))
   remoteVersion = VmomiSupport.versionMap.get(versionKey, None)
   if not remoteVersion:
      return None
   if VmomiSupport.IsChildVersion(localVersion, remoteVersion):
      return remoteVersion

# Get the VMODL version by checking the existence of vSAN namespace.
def GetLatestVmodlVersion(hostname, port=443):
   try:
      if valid_ipv6(hostname):
         hostname = "[%s]" % hostname
      vsanVmodlUrl = 'https://%s:%s/sdk/vsanServiceVersions.xml' % (hostname, str(port))
      if (hasattr(ssl, '_create_unverified_context') and
         hasattr(ssl, '_create_default_https_context')):
         ssl._create_default_https_context = ssl._create_unverified_context
      localVersion = VmomiSupport.newestVersions.GetName("vsan")
      xmldoc = minidom.parse(urlopen(vsanVmodlUrl, timeout=5))
      for element in xmldoc.getElementsByTagName('name'):
         if (element.firstChild.nodeValue == "urn:vsan"):
            versions = xmldoc.getElementsByTagName('version')
            for ver in versions:
               versionId = ver.firstChild.nodeValue
               version = getVsanVersionFromNamespace(versionId, localVersion)
               if version:
                  return version
            return localVersion
         else:
            return VmomiSupport.newestVersions.GetName('vim')
   except Exception as e:
      # Any exception like failing to open the XML or failed to parse the
      # the content should lead to the returning of namespace with vim.
      return VmomiSupport.newestVersions.GetName('vim')




# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vim.host import ScsiDisk

from pyVmomi.vim.vm import DiskDeviceInfo

class ScsiDiskDeviceInfo(DiskDeviceInfo):
   disk: Optional[ScsiDisk] = None
   transportHint: Optional[str] = None
   lunNumber: Optional[int] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vim.profile import ApplyProfile

from pyVmomi.vim.profile.host import AuthenticationProfile
from pyVmomi.vim.profile.host import DateTimeProfile
from pyVmomi.vim.profile.host import FirewallProfile
from pyVmomi.vim.profile.host import HostMemoryProfile
from pyVmomi.vim.profile.host import NetworkProfile
from pyVmomi.vim.profile.host import OptionProfile
from pyVmomi.vim.profile.host import SecurityProfile
from pyVmomi.vim.profile.host import ServiceProfile
from pyVmomi.vim.profile.host import StorageProfile
from pyVmomi.vim.profile.host import UserGroupProfile
from pyVmomi.vim.profile.host import UserProfile

class HostApplyProfile(ApplyProfile):
   memory: Optional[HostMemoryProfile] = None
   storage: Optional[StorageProfile] = None
   network: Optional[NetworkProfile] = None
   datetime: Optional[DateTimeProfile] = None
   firewall: Optional[FirewallProfile] = None
   security: Optional[SecurityProfile] = None
   service: list[ServiceProfile] = []
   option: list[OptionProfile] = []
   userAccount: list[UserProfile] = []
   usergroupAccount: list[UserGroupProfile] = []
   authentication: Optional[AuthenticationProfile] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import Optional

from pyVmomi.VmomiSupport import Enum
from pyVmomi.VmomiSupport import long

from pyVmomi.vim import SharesInfo
from pyVmomi.vim import StorageResourceManager

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.encryption import CryptoKeyId

from pyVmomi.vim.vm import BaseIndependentFilterSpec

from pyVmomi.vim.vslm import ID

from pyVmomi.vim.vm.device import VirtualDevice

class VirtualDisk(VirtualDevice):
   class DeltaDiskFormat(Enum):
      redoLogFormat: ClassVar['DeltaDiskFormat'] = 'redoLogFormat'
      nativeFormat: ClassVar['DeltaDiskFormat'] = 'nativeFormat'
      seSparseFormat: ClassVar['DeltaDiskFormat'] = 'seSparseFormat'

   class DeltaDiskFormatVariant(Enum):
      vmfsSparseVariant: ClassVar['DeltaDiskFormatVariant'] = 'vmfsSparseVariant'
      vsanSparseVariant: ClassVar['DeltaDiskFormatVariant'] = 'vsanSparseVariant'

   class Sharing(Enum):
      sharingNone: ClassVar['Sharing'] = 'sharingNone'
      sharingMultiWriter: ClassVar['Sharing'] = 'sharingMultiWriter'

   class SparseVer1BackingInfo(VirtualDevice.FileBackingInfo):
      diskMode: str
      split: Optional[bool] = None
      writeThrough: Optional[bool] = None
      spaceUsedInKB: Optional[long] = None
      contentId: Optional[str] = None
      parent: Optional[SparseVer1BackingInfo] = None

   class SparseVer2BackingInfo(VirtualDevice.FileBackingInfo):
      diskMode: str
      split: Optional[bool] = None
      writeThrough: Optional[bool] = None
      spaceUsedInKB: Optional[long] = None
      uuid: Optional[str] = None
      contentId: Optional[str] = None
      changeId: Optional[str] = None
      parent: Optional[SparseVer2BackingInfo] = None
      keyId: Optional[CryptoKeyId] = None

   class FlatVer1BackingInfo(VirtualDevice.FileBackingInfo):
      diskMode: str
      split: Optional[bool] = None
      writeThrough: Optional[bool] = None
      contentId: Optional[str] = None
      parent: Optional[FlatVer1BackingInfo] = None

   class FlatVer2BackingInfo(VirtualDevice.FileBackingInfo):
      diskMode: str
      split: Optional[bool] = None
      writeThrough: Optional[bool] = None
      thinProvisioned: Optional[bool] = None
      eagerlyScrub: Optional[bool] = None
      uuid: Optional[str] = None
      contentId: Optional[str] = None
      changeId: Optional[str] = None
      parent: Optional[FlatVer2BackingInfo] = None
      deltaDiskFormat: Optional[str] = None
      digestEnabled: Optional[bool] = None
      deltaGrainSize: Optional[int] = None
      deltaDiskFormatVariant: Optional[str] = None
      sharing: Optional[str] = None
      keyId: Optional[CryptoKeyId] = None

   class SeSparseBackingInfo(VirtualDevice.FileBackingInfo):
      diskMode: str
      writeThrough: Optional[bool] = None
      uuid: Optional[str] = None
      contentId: Optional[str] = None
      changeId: Optional[str] = None
      parent: Optional[SeSparseBackingInfo] = None
      deltaDiskFormat: Optional[str] = None
      digestEnabled: Optional[bool] = None
      grainSize: Optional[int] = None
      keyId: Optional[CryptoKeyId] = None

   class RawDiskVer2BackingInfo(VirtualDevice.DeviceBackingInfo):
      descriptorFileName: str
      uuid: Optional[str] = None
      changeId: Optional[str] = None
      sharing: Optional[str] = None

   class PartitionedRawDiskVer2BackingInfo(RawDiskVer2BackingInfo):
      partition: list[int] = []

   class RawDiskMappingVer1BackingInfo(VirtualDevice.FileBackingInfo):
      lunUuid: Optional[str] = None
      deviceName: Optional[str] = None
      compatibilityMode: Optional[str] = None
      diskMode: Optional[str] = None
      uuid: Optional[str] = None
      contentId: Optional[str] = None
      changeId: Optional[str] = None
      parent: Optional[RawDiskMappingVer1BackingInfo] = None
      deltaDiskFormat: Optional[str] = None
      deltaGrainSize: Optional[int] = None
      sharing: Optional[str] = None

   class LocalPMemBackingInfo(VirtualDevice.FileBackingInfo):
      diskMode: str
      uuid: Optional[str] = None
      volumeUUID: Optional[str] = None
      contentId: Optional[str] = None

   class VFlashCacheConfigInfo(DynamicData):
      class CacheConsistencyType(Enum):
         strong: ClassVar['CacheConsistencyType'] = 'strong'
         weak: ClassVar['CacheConsistencyType'] = 'weak'

      class CacheMode(Enum):
         write_thru: ClassVar['CacheMode'] = 'write_thru'
         write_back: ClassVar['CacheMode'] = 'write_back'

      vFlashModule: Optional[str] = None
      reservationInMB: Optional[long] = None
      cacheConsistencyType: Optional[str] = None
      cacheMode: Optional[str] = None
      blockSizeInKB: Optional[long] = None

   capacityInKB: long
   capacityInBytes: Optional[long] = None
   shares: Optional[SharesInfo] = None
   storageIOAllocation: Optional[StorageResourceManager.IOAllocationInfo] = None
   diskObjectId: Optional[str] = None
   vFlashCacheConfigInfo: Optional[VFlashCacheConfigInfo] = None
   iofilter: list[str] = []
   vDiskId: Optional[ID] = None
   vDiskVersion: Optional[int] = None
   virtualDiskFormat: Optional[str] = None
   nativeUnmanagedLinkedClone: Optional[bool] = None
   independentFilters: list[BaseIndependentFilterSpec] = []
   guestReadOnly: Optional[bool] = None



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from datetime import datetime

from typing import NoReturn
from typing import Optional

from pyVmomi.VmomiSupport import ManagedObject
from pyVmomi.VmomiSupport import long

from pyVmomi.vim import VirtualMachine

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.vm.guest import GuestAuthentication

class ProcessManager(ManagedObject):
   class ProgramSpec(DynamicData):
      programPath: str
      arguments: str
      workingDirectory: Optional[str] = None
      envVariables: list[str] = []

   class WindowsProgramSpec(ProgramSpec):
      startMinimized: bool

   class ProcessInfo(DynamicData):
      name: str
      pid: long
      owner: str
      cmdLine: str
      startTime: datetime
      endTime: Optional[datetime] = None
      exitCode: Optional[int] = None

   def StartProgram(self, vm: VirtualMachine, auth: GuestAuthentication, spec: ProgramSpec) -> long: ...
   def ListProcesses(self, vm: VirtualMachine, auth: GuestAuthentication, pids: list[long]) -> list[ProcessInfo]: ...
   def TerminateProcess(self, vm: VirtualMachine, auth: GuestAuthentication, pid: long) -> NoReturn: ...
   def ReadEnvironmentVariable(self, vm: VirtualMachine, auth: GuestAuthentication, names: list[str]) -> list[str]: ...



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import Optional

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vmodl import DynamicData

class GraphicsConfig(DynamicData):
   class GraphicsType(Enum):
      shared: ClassVar['GraphicsType'] = 'shared'
      sharedDirect: ClassVar['GraphicsType'] = 'sharedDirect'

   class SharedPassthruAssignmentPolicy(Enum):
      performance: ClassVar['SharedPassthruAssignmentPolicy'] = 'performance'
      consolidation: ClassVar['SharedPassthruAssignmentPolicy'] = 'consolidation'

   class VgpuMode(Enum):
      sameSize: ClassVar['VgpuMode'] = 'sameSize'
      mixedSize: ClassVar['VgpuMode'] = 'mixedSize'

   class DeviceType(DynamicData):
      deviceId: str
      graphicsType: str
      vgpuMode: Optional[str] = None

   hostDefaultGraphicsType: str
   sharedPassthruAssignmentPolicy: str
   deviceType: list[DeviceType] = []



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import Optional

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.cluster import DasConfigInfo
from pyVmomi.vim.cluster import DasVmConfigSpec
from pyVmomi.vim.cluster import DrsConfigInfo
from pyVmomi.vim.cluster import DrsVmConfigSpec
from pyVmomi.vim.cluster import RuleSpec

class ConfigSpec(DynamicData):
   dasConfig: Optional[DasConfigInfo] = None
   dasVmConfigSpec: list[DasVmConfigSpec] = []
   drsConfig: Optional[DrsConfigInfo] = None
   drsVmConfigSpec: list[DrsVmConfigSpec] = []
   rulesSpec: list[RuleSpec] = []



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from pyVmomi.vim.option import BoolOption
from pyVmomi.vim.option import IntOption

from pyVmomi.vim.vm.device import VirtualControllerOption
from pyVmomi.vim.vm.device import VirtualSCSIController

class VirtualSCSIControllerOption(VirtualControllerOption):
   numSCSIDisks: IntOption
   numSCSICdroms: IntOption
   numSCSIPassthrough: IntOption
   sharing: list[VirtualSCSIController.Sharing] = []
   defaultSharedIndex: int
   hotAddRemove: BoolOption
   scsiCtlrUnitNumber: int



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import ClassVar
from typing import Optional

from pyVmomi.VmomiSupport import Enum

from pyVmomi.vmodl import DynamicData

from pyVmomi.vim.host import FeatureCapability
from pyVmomi.vim.host import PhysicalNic
from pyVmomi.vim.host import VirtualNic

from pyVmomi.vim.option import OptionValue

class OpaqueSwitch(DynamicData):
   class OpaqueSwitchState(Enum):
      up: ClassVar['OpaqueSwitchState'] = 'up'
      warning: ClassVar['OpaqueSwitchState'] = 'warning'
      down: ClassVar['OpaqueSwitchState'] = 'down'
      maintenance: ClassVar['OpaqueSwitchState'] = 'maintenance'

   class PhysicalNicZone(DynamicData):
      key: str
      pnicDevice: list[str] = []

   key: str
   name: Optional[str] = None
   pnic: list[PhysicalNic] = []
   pnicZone: list[PhysicalNicZone] = []
   status: Optional[str] = None
   vtep: list[VirtualNic] = []
   extraConfig: list[OptionValue] = []
   featureCapability: list[FeatureCapability] = []



# Copyright (c) 2006-2025 Broadcom. All Rights Reserved.
# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
# and/or its subsidiaries.

# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******

from typing import NoReturn
from typing import Optional

from pyVmomi.VmomiSupport import ManagedObject

from pyVmomi.vim import ClusterComputeResource
from pyVmomi.vim import HealthUpdate
from pyVmomi.vim import HealthUpdateInfo
from pyVmomi.vim import HostSystem
from pyVmomi.vim import ManagedEntity

class HealthUpdateManager(ManagedObject):
   def RegisterProvider(self, name: str, healthUpdateInfo: list[HealthUpdateInfo]) -> str: ...
   def UnregisterProvider(self, providerId: str) -> NoReturn: ...
   def QueryProviderList(self) -> list[str]: ...
   def HasProvider(self, id: str) -> bool: ...
   def QueryProviderName(self, id: str) -> str: ...
   def QueryHealthUpdateInfos(self, providerId: str) -> list[HealthUpdateInfo]: ...
   def AddMonitoredEntities(self, providerId: str, entities: list[ManagedEntity]) -> NoReturn: ...
   def RemoveMonitoredEntities(self, providerId: str, entities: list[ManagedEntity]) -> NoReturn: ...
   def QueryMonitoredEntities(self, providerId: str) -> list[ManagedEntity]: ...
   def HasMonitoredEntity(self, providerId: str, entity: ManagedEntity) -> bool: ...
   def QueryUnmonitoredHosts(self, providerId: str, cluster: ClusterComputeResource) -> list[HostSystem]: ...
   def PostHealthUpdates(self, providerId: str, updates: list[HealthUpdate]) -> NoReturn: ...
   def QueryHealthUpdates(self, providerId: str) -> list[HealthUpdate]: ...
   def AddFilter(self, providerId: str, filterName: str, infoIds: list[str]) -> str: ...
   def QueryFilterList(self, providerId: str) -> list[str]: ...
   def QueryFilterName(self, filterId: str) -> str: ...
   def QueryFilterInfoIds(self, filterId: str) -> list[str]: ...
   def QueryFilterEntities(self, filterId: str) -> list[ManagedEntity]: ...
   def AddFilterEntities(self, filterId: str, entities: list[ManagedEntity]) -> NoReturn: ...
   def RemoveFilterEntities(self, filterId: str, entities: list[ManagedEntity]) -> NoReturn: ...
   def RemoveFilter(self, filterId: str) -> NoReturn: ...
